var Common;
(function (Common) {
    (function (Ensure) {
        function isStrict(obj, func, propertyName, message) {
            if(!func(obj)) {
                var msg = "";
                if(!isBlank(propertyName)) {
                    msg += "[" + propertyName + "]: ";
                }
                if(!isBlank(message)) {
                    msg += message;
                }
                var c = console;
                c.trace();
                throw msg;
            }
            return Ensure;
        }
        function defined(obj, propertyName, message) {
            if (typeof propertyName === "undefined") { propertyName = ""; }
            if (typeof message === "undefined") { message = "Property cannot be undefined or null!"; }
            return isStrict(obj, function (o) {
                return isDefined(o) && isNotNull(o);
            }, propertyName, message);
        }
        Ensure.defined = defined;
        function notBlank(text, propertyName, message) {
            return isStrict(text, function (obj) {
                return !isBlank(obj);
            }, propertyName, message);
        }
        Ensure.notBlank = notBlank;
        function hasValue(obj) {
            return Ensure.isDefined(obj) && Ensure.isNotNull(obj);
        }
        Ensure.hasValue = hasValue;
        function isNotNull(obj) {
            return !Ensure.isNull(obj);
        }
        Ensure.isNotNull = isNotNull;
        function isNull(obj) {
            return obj === null;
        }
        Ensure.isNull = isNull;
        function isDefined(obj) {
            return obj !== undefined;
        }
        Ensure.isDefined = isDefined;
        function isUndefined(obj) {
            return !Ensure.isDefined(obj);
        }
        Ensure.isUndefined = isUndefined;
        function isFunction(obj) {
            return typeof (obj) === "function";
        }
        Ensure.isFunction = isFunction;
        function isBlank(text) {
            return !hasValue(text) || text === "";
        }
        Ensure.isBlank = isBlank;
        function toIntervallum(num, min, max) {
            if (typeof min === "undefined") { min = 0; }
            if (typeof max === "undefined") { max = 1; }
            Ensure.defined(min).defined(max).defined(num);
            if(num < min) {
                num = min;
            } else if(num > max) {
                num = max;
            }
            return num;
        }
        Ensure.toIntervallum = toIntervallum;
    })(Common.Ensure || (Common.Ensure = {}));
    var Ensure = Common.Ensure;
        var SafeElement = (function () {
        function SafeElement(object) {
            this._object = null;
            this._object = object;
        }
        SafeElement.prototype.getObject = function () {
            return this._object;
        };
        SafeElement.prototype.has = function () {
            var _this = this;
            var property = [];
            for (var _i = 0; _i < (arguments.length - 0); _i++) {
                property[_i] = arguments[_i + 0];
            }
            return Enumerable.From(property).All(function (value) {
                return Ensure.hasValue(_this._object[value]);
            });
        };
        SafeElement.prototype.apply = function (func) {
            return new SafeElement(func(this._object));
        };
        SafeElement.prototype.tryGet = function (property, func) {
            if(this.has(property)) {
                func(this._object[property], this._object);
            }
            return this;
        };
        return SafeElement;
    })();    
    function Safe(object) {
        Ensure.defined(object);
        return new SafeElement(object);
    }
    Common.Safe = Safe;
    var LetElement = (function () {
        function LetElement(object) {
            this._object = object;
        }
        LetElement.prototype.default = function (value) {
            var result = this._object;
            if(Ensure.isNull(result) || Ensure.isUndefined(result)) {
                result = value;
            }
            return new LetElement(result);
        };
        LetElement.prototype.be = function (func) {
            return new LetElement(func(this._object));
        };
        Object.defineProperty(LetElement.prototype, "value", {
            get: function () {
                return this._object;
            },
            enumerable: true,
            configurable: true
        });
        return LetElement;
    })();    
    var LazyLetElement = (function () {
        function LazyLetElement(object, func) {
            if (typeof func === "undefined") { func = function (x) {
                return x;
            }; }
            this._object = object;
            this._func = func;
        }
        LazyLetElement.prototype.default = function (value) {
            var _this = this;
            return new LazyLetElement(this._object, function (x) {
                var result = _this._func(x);
                if(Ensure.isNull(result) || Ensure.isUndefined(result)) {
                    result = value;
                }
                return result;
            });
        };
        LazyLetElement.prototype.be = function (func) {
            var _this = this;
            return new LazyLetElement(this._object, function (value) {
                return func(_this._func(value));
            });
        };
        Object.defineProperty(LazyLetElement.prototype, "value", {
            get: function () {
                return this._func(this._object);
            },
            enumerable: true,
            configurable: true
        });
        return LazyLetElement;
    })();    
    function Let(object, lazy) {
        if (typeof lazy === "undefined") { lazy = false; }
        if(lazy) {
            return new LazyLetElement(object);
        } else {
            return new LetElement(object);
        }
    }
    Common.Let = Let;
    function getClass(object) {
        return object.constructor;
    }
    Common.getClass = getClass;
    ;
    Common.FunctionObject = function () {
        var funcs = [];
        for (var _i = 0; _i < (arguments.length - 0); _i++) {
            funcs[_i] = arguments[_i + 0];
        }
        var _functionList = funcs || [];
        var that = function (params) {
            var results = [];
            for(var ind in _functionList) {
                results.push(_functionList[ind](params));
            }
            return results;
        };
        that.elements = _functionList;
        that.push = function () {
            var items = [];
            for (var _i = 0; _i < (arguments.length - 0); _i++) {
                items[_i] = arguments[_i + 0];
            }
            for(var ind in items) {
                _functionList.push(items[ind]);
            }
            return that;
        };
        that.pop = function () {
            return _functionList.pop();
        };
        that.clear = function () {
            _functionList = [];
            return that;
        };
        return that;
    };
    Common.property = function (options) {
        if(!options) {
            return undefined;
        }
        if(options.auto) {
            var _value = options.default ? options.default() : undefined;
            return function (value) {
                if(!value) {
                    return _value;
                } else {
                    _value = value;
                    return options.that;
                }
            };
        } else {
            return function (value) {
                if(!value && options.get) {
                    if(!options.get() && options.default) {
                        options.set(options.default());
                    }
                    return options.get();
                } else if(options.set) {
                    return options.set(value) || options.that;
                }
            };
        }
    };
})(Common || (Common = {}));
var Let = Common.Let;
var Ensure = Common.Ensure;
var Safe = Common.Safe;
var FunctionObject = Common.FunctionObject;
//@ sourceMappingURL=Common.js.map
