var __extends = this.__extends || function (d, b) {
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
var CodeCompass;
(function (CodeCompass) {
    (function (Drawings) {
        var Color = (function () {
            function Color(red, green, blue, alpha) {
                if (typeof alpha === "undefined") { alpha = 1; }
                this._R = Ensure.toIntervallum(red, 0, 255);
                this._G = Ensure.toIntervallum(green, 0, 255);
                this._B = Ensure.toIntervallum(blue, 0, 255);
                this._A = Ensure.toIntervallum(alpha, 0, 1);
            }
            Object.defineProperty(Color.prototype, "R", {
                get: function () {
                    return this._R;
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Color.prototype, "G", {
                get: function () {
                    return this._G;
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Color.prototype, "B", {
                get: function () {
                    return this._B;
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Color.prototype, "A", {
                get: function () {
                    return this._A;
                },
                enumerable: true,
                configurable: true
            });
            Color.prototype.add = function (color) {
                return new Color(this.R + color.R, this.G + color.G, this.B + color.B, this.A + color.A);
            };
            Color.prototype.toCSS = function () {
                return "rgba(" + [
                    this.R, 
                    this.G, 
                    this.B, 
                    this.A
                ].join(",") + ")";
            };
            Color.prototype.toHex = function () {
                var hexCodes = [
                    "0", 
                    "1", 
                    "2", 
                    "3", 
                    "4", 
                    "5", 
                    "6", 
                    "7", 
                    "8", 
                    "9", 
                    "a", 
                    "b", 
                    "c", 
                    "d", 
                    "e", 
                    "f"
                ];
                var result = "#";
                [
                    this.R, 
                    this.G, 
                    this.B, 
                    this.A * 255
                ].forEach(function (value, index, array) {
                    if(value == 0) {
                        result += "00";
                    }
                    while(value != 0) {
                        result += hexCodes[value % 16];
                        value = Math.floor(value / 16);
                    }
                });
                return result;
            };
            return Color;
        })();
        Drawings.Color = Color;        
        var Size = (function () {
            function Size(width, height) {
                this.width = width;
                this.height = height;
            }
            Object.defineProperty(Size, "Zero", {
                get: function () {
                    return new Size(0, 0);
                },
                enumerable: true,
                configurable: true
            });
            return Size;
        })();
        Drawings.Size = Size;        
        var Point2D = (function () {
            function Point2D(x, y) {
                if (typeof x === "undefined") { x = 0; }
                if (typeof y === "undefined") { y = 0; }
                Ensure.defined(x, "x: number (Point2D)");
                Ensure.defined(y, "y: number (Point2D)");
                this._x = x;
                this._y = y;
            }
            Object.defineProperty(Point2D.prototype, "x", {
                get: function () {
                    return this._x;
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Point2D.prototype, "y", {
                get: function () {
                    return this._y;
                },
                enumerable: true,
                configurable: true
            });
            Point2D.create = function create(x, y) {
                if (typeof x === "undefined") { x = 0; }
                if (typeof y === "undefined") { y = 0; }
                return new Point2D(x, y);
            };
            Object.defineProperty(Point2D, "Origo", {
                get: function () {
                    return new Point2D(0, 0);
                },
                enumerable: true,
                configurable: true
            });
            Point2D.prototype.add = function (point) {
                Ensure.defined(point, "point", "Point cannot be null or undefined!");
                return new Point2D(this.x + point.x, this.y + point.y);
            };
            Point2D.prototype.distance = function (point) {
                if (typeof point === "undefined") { point = Point2D.Origo; }
                Ensure.defined(point, "point: IPoint2D (Point2D)", "Point cannot be null or undefined!");
                var sqr = function (x) {
                    return x * x;
                };
                return Math.sqrt(sqr(this.x - point.x) + sqr(this.y - point.y));
            };
            Point2D.prototype.negate = function () {
                return new Point2D(-this.x, -this.y);
            };
            return Point2D;
        })();
        Drawings.Point2D = Point2D;        
        var RelativePoint2D = (function (_super) {
            __extends(RelativePoint2D, _super);
            function RelativePoint2D(x, y, isAbsolute, horizontalAlign, verticalAlign) {
                if (typeof x === "undefined") { x = 0; }
                if (typeof y === "undefined") { y = 0; }
                if (typeof isAbsolute === "undefined") { isAbsolute = false; }
                if (typeof horizontalAlign === "undefined") { horizontalAlign = HorizontalAlign.left; }
                if (typeof verticalAlign === "undefined") { verticalAlign = VerticalAlign.top; }
                        _super.call(this, x, y);
                this.isAbsolute = isAbsolute;
                this.horizontalAlign = horizontalAlign;
                this.verticalAlign = verticalAlign;
            }
            return RelativePoint2D;
        })(Point2D);
        Drawings.RelativePoint2D = RelativePoint2D;        
        var RelativeSize = (function (_super) {
            __extends(RelativeSize, _super);
            function RelativeSize(width, height, isAbsolute) {
                if (typeof width === "undefined") { width = 0; }
                if (typeof height === "undefined") { height = 0; }
                if (typeof isAbsolute === "undefined") { isAbsolute = false; }
                        _super.call(this, width, height);
                this.width = width;
                this.height = height;
                this.isAbsolute = isAbsolute;
            }
            Object.defineProperty(RelativeSize, "Zero", {
                get: function () {
                    return new RelativeSize(0, 0);
                },
                enumerable: true,
                configurable: true
            });
            return RelativeSize;
        })(Size);
        Drawings.RelativeSize = RelativeSize;        
        (function (HorizontalAlign) {
            HorizontalAlign._map = [];
            HorizontalAlign._map[0] = "left";
            HorizontalAlign.left = 0;
            HorizontalAlign._map[1] = "center";
            HorizontalAlign.center = 1;
            HorizontalAlign._map[2] = "right";
            HorizontalAlign.right = 2;
        })(Drawings.HorizontalAlign || (Drawings.HorizontalAlign = {}));
        var HorizontalAlign = Drawings.HorizontalAlign;
        (function (VerticalAlign) {
            VerticalAlign._map = [];
            VerticalAlign._map[0] = "top";
            VerticalAlign.top = 0;
            VerticalAlign._map[1] = "center";
            VerticalAlign.center = 1;
            VerticalAlign._map[2] = "bottom";
            VerticalAlign.bottom = 2;
        })(Drawings.VerticalAlign || (Drawings.VerticalAlign = {}));
        var VerticalAlign = Drawings.VerticalAlign;
        (function (ArrowEnd) {
            ArrowEnd._map = [];
            ArrowEnd._map[0] = "classic";
            ArrowEnd.classic = 0;
            ArrowEnd._map[1] = "block";
            ArrowEnd.block = 1;
            ArrowEnd._map[2] = "open";
            ArrowEnd.open = 2;
            ArrowEnd._map[3] = "oval";
            ArrowEnd.oval = 3;
            ArrowEnd._map[4] = "diamond";
            ArrowEnd.diamond = 4;
            ArrowEnd._map[5] = "none";
            ArrowEnd.none = 5;
        })(Drawings.ArrowEnd || (Drawings.ArrowEnd = {}));
        var ArrowEnd = Drawings.ArrowEnd;
        ;
        (function (LineCap) {
            LineCap._map = [];
            LineCap._map[0] = "butt";
            LineCap.butt = 0;
            LineCap._map[1] = "square";
            LineCap.square = 1;
            LineCap._map[2] = "round";
            LineCap.round = 2;
        })(Drawings.LineCap || (Drawings.LineCap = {}));
        var LineCap = Drawings.LineCap;
        (function (LineJoin) {
            LineJoin._map = [];
            LineJoin._map[0] = "bevel";
            LineJoin.bevel = 0;
            LineJoin._map[1] = "round";
            LineJoin.round = 1;
            LineJoin._map[2] = "miter";
            LineJoin.miter = 2;
        })(Drawings.LineJoin || (Drawings.LineJoin = {}));
        var LineJoin = Drawings.LineJoin;
    })(CodeCompass.Drawings || (CodeCompass.Drawings = {}));
    var Drawings = CodeCompass.Drawings;
})(CodeCompass || (CodeCompass = {}));
var CodeCompass;
(function (CodeCompass) {
    (function (Drawings) {
            })(CodeCompass.Drawings || (CodeCompass.Drawings = {}));
    var Drawings = CodeCompass.Drawings;
})(CodeCompass || (CodeCompass = {}));
var CodeCompass;
(function (CodeCompass) {
    (function (Drawings) {
        var Colors = (function () {
            function Colors() { }
            Object.defineProperty(Colors, "AliceBlue", {
                get: function () {
                    return new Drawings.Color(240, 248, 255, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "AntiqueWhite", {
                get: function () {
                    return new Drawings.Color(250, 235, 215, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Aqua", {
                get: function () {
                    return new Drawings.Color(0, 255, 255, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Aquamarine", {
                get: function () {
                    return new Drawings.Color(127, 255, 212, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Azure", {
                get: function () {
                    return new Drawings.Color(240, 255, 255, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Beige", {
                get: function () {
                    return new Drawings.Color(245, 245, 220, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Bisque", {
                get: function () {
                    return new Drawings.Color(255, 228, 196, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Black", {
                get: function () {
                    return new Drawings.Color(0, 0, 0, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "BlanchedAlmond", {
                get: function () {
                    return new Drawings.Color(255, 235, 205, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Blue", {
                get: function () {
                    return new Drawings.Color(0, 0, 255, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "BlueViolet", {
                get: function () {
                    return new Drawings.Color(138, 43, 226, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Brown", {
                get: function () {
                    return new Drawings.Color(165, 42, 42, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "BurlyWood", {
                get: function () {
                    return new Drawings.Color(222, 184, 135, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "CadetBlue", {
                get: function () {
                    return new Drawings.Color(95, 158, 160, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Chartreuse", {
                get: function () {
                    return new Drawings.Color(127, 255, 0, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Chocolate", {
                get: function () {
                    return new Drawings.Color(210, 105, 30, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Coral", {
                get: function () {
                    return new Drawings.Color(255, 127, 80, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "CornflowerBlue", {
                get: function () {
                    return new Drawings.Color(100, 149, 237, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Cornsilk", {
                get: function () {
                    return new Drawings.Color(255, 248, 220, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Crimson", {
                get: function () {
                    return new Drawings.Color(220, 20, 60, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Cyan", {
                get: function () {
                    return new Drawings.Color(0, 255, 255, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "DarkBlue", {
                get: function () {
                    return new Drawings.Color(0, 0, 139, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "DarkCyan", {
                get: function () {
                    return new Drawings.Color(0, 139, 139, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "DarkGoldenrod", {
                get: function () {
                    return new Drawings.Color(184, 134, 11, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "DarkGray", {
                get: function () {
                    return new Drawings.Color(169, 169, 169, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "DarkGreen", {
                get: function () {
                    return new Drawings.Color(0, 100, 0, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "DarkKhaki", {
                get: function () {
                    return new Drawings.Color(189, 183, 107, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "DarkMagenta", {
                get: function () {
                    return new Drawings.Color(139, 0, 139, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "DarkOliveGreen", {
                get: function () {
                    return new Drawings.Color(85, 107, 47, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "DarkOrange", {
                get: function () {
                    return new Drawings.Color(255, 140, 0, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "DarkOrchid", {
                get: function () {
                    return new Drawings.Color(153, 50, 204, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "DarkRed", {
                get: function () {
                    return new Drawings.Color(139, 0, 0, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "DarkSalmon", {
                get: function () {
                    return new Drawings.Color(233, 150, 122, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "DarkSeaGreen", {
                get: function () {
                    return new Drawings.Color(143, 188, 143, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "DarkSlateBlue", {
                get: function () {
                    return new Drawings.Color(72, 61, 139, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "DarkSlateGray", {
                get: function () {
                    return new Drawings.Color(47, 79, 79, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "DarkTurquoise", {
                get: function () {
                    return new Drawings.Color(0, 206, 209, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "DarkViolet", {
                get: function () {
                    return new Drawings.Color(148, 0, 211, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "DeepPink", {
                get: function () {
                    return new Drawings.Color(255, 20, 147, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "DeepSkyBlue", {
                get: function () {
                    return new Drawings.Color(0, 191, 255, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "DimGray", {
                get: function () {
                    return new Drawings.Color(105, 105, 105, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "DodgerBlue", {
                get: function () {
                    return new Drawings.Color(30, 144, 255, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Firebrick", {
                get: function () {
                    return new Drawings.Color(178, 34, 34, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "FloralWhite", {
                get: function () {
                    return new Drawings.Color(255, 250, 240, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "ForestGreen", {
                get: function () {
                    return new Drawings.Color(34, 139, 34, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Fuchsia", {
                get: function () {
                    return new Drawings.Color(255, 0, 255, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Gainsboro", {
                get: function () {
                    return new Drawings.Color(220, 220, 220, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "GhostWhite", {
                get: function () {
                    return new Drawings.Color(248, 248, 255, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Gold", {
                get: function () {
                    return new Drawings.Color(255, 215, 0, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Goldenrod", {
                get: function () {
                    return new Drawings.Color(218, 165, 32, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Gray", {
                get: function () {
                    return new Drawings.Color(128, 128, 128, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Green", {
                get: function () {
                    return new Drawings.Color(0, 128, 0, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "GreenYellow", {
                get: function () {
                    return new Drawings.Color(173, 255, 47, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Honeydew", {
                get: function () {
                    return new Drawings.Color(240, 255, 240, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "HotPink", {
                get: function () {
                    return new Drawings.Color(255, 105, 180, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "IndianRed", {
                get: function () {
                    return new Drawings.Color(205, 92, 92, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Indigo", {
                get: function () {
                    return new Drawings.Color(75, 0, 130, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Ivory", {
                get: function () {
                    return new Drawings.Color(255, 255, 240, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Khaki", {
                get: function () {
                    return new Drawings.Color(240, 230, 140, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Lavender", {
                get: function () {
                    return new Drawings.Color(230, 230, 250, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "LavenderBlush", {
                get: function () {
                    return new Drawings.Color(255, 240, 245, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "LawnGreen", {
                get: function () {
                    return new Drawings.Color(124, 252, 0, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "LemonChiffon", {
                get: function () {
                    return new Drawings.Color(255, 250, 205, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "LightBlue", {
                get: function () {
                    return new Drawings.Color(173, 216, 230, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "LightCoral", {
                get: function () {
                    return new Drawings.Color(240, 128, 128, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "LightCyan", {
                get: function () {
                    return new Drawings.Color(224, 255, 255, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "LightGoldenrodYellow", {
                get: function () {
                    return new Drawings.Color(250, 250, 210, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "LightGray", {
                get: function () {
                    return new Drawings.Color(211, 211, 211, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "LightGreen", {
                get: function () {
                    return new Drawings.Color(144, 238, 144, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "LightPink", {
                get: function () {
                    return new Drawings.Color(255, 182, 193, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "LightSalmon", {
                get: function () {
                    return new Drawings.Color(255, 160, 122, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "LightSeaGreen", {
                get: function () {
                    return new Drawings.Color(32, 178, 170, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "LightSkyBlue", {
                get: function () {
                    return new Drawings.Color(135, 206, 250, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "LightSlateGray", {
                get: function () {
                    return new Drawings.Color(119, 136, 153, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "LightSteelBlue", {
                get: function () {
                    return new Drawings.Color(176, 196, 222, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "LightYellow", {
                get: function () {
                    return new Drawings.Color(255, 255, 224, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Lime", {
                get: function () {
                    return new Drawings.Color(0, 255, 0, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "LimeGreen", {
                get: function () {
                    return new Drawings.Color(50, 205, 50, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Linen", {
                get: function () {
                    return new Drawings.Color(250, 240, 230, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Magenta", {
                get: function () {
                    return new Drawings.Color(255, 0, 255, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Maroon", {
                get: function () {
                    return new Drawings.Color(128, 0, 0, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "MediumAquamarine", {
                get: function () {
                    return new Drawings.Color(102, 205, 170, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "MediumBlue", {
                get: function () {
                    return new Drawings.Color(0, 0, 205, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "MediumOrchid", {
                get: function () {
                    return new Drawings.Color(186, 85, 211, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "MediumPurple", {
                get: function () {
                    return new Drawings.Color(147, 112, 219, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "MediumSeaGreen", {
                get: function () {
                    return new Drawings.Color(60, 179, 113, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "MediumSlateBlue", {
                get: function () {
                    return new Drawings.Color(123, 104, 238, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "MediumSpringGreen", {
                get: function () {
                    return new Drawings.Color(0, 250, 154, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "MediumTurquoise", {
                get: function () {
                    return new Drawings.Color(72, 209, 204, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "MediumVioletRed", {
                get: function () {
                    return new Drawings.Color(199, 21, 133, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "MidnightBlue", {
                get: function () {
                    return new Drawings.Color(25, 25, 112, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "MintCream", {
                get: function () {
                    return new Drawings.Color(245, 255, 250, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "MistyRose", {
                get: function () {
                    return new Drawings.Color(255, 228, 225, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Moccasin", {
                get: function () {
                    return new Drawings.Color(255, 228, 181, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "NavajoWhite", {
                get: function () {
                    return new Drawings.Color(255, 222, 173, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Navy", {
                get: function () {
                    return new Drawings.Color(0, 0, 128, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "OldLace", {
                get: function () {
                    return new Drawings.Color(253, 245, 230, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Olive", {
                get: function () {
                    return new Drawings.Color(128, 128, 0, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "OliveDrab", {
                get: function () {
                    return new Drawings.Color(107, 142, 35, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Orange", {
                get: function () {
                    return new Drawings.Color(255, 165, 0, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "OrangeRed", {
                get: function () {
                    return new Drawings.Color(255, 69, 0, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Orchid", {
                get: function () {
                    return new Drawings.Color(218, 112, 214, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "PaleGoldenrod", {
                get: function () {
                    return new Drawings.Color(238, 232, 170, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "PaleGreen", {
                get: function () {
                    return new Drawings.Color(152, 251, 152, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "PaleTurquoise", {
                get: function () {
                    return new Drawings.Color(175, 238, 238, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "PaleVioletRed", {
                get: function () {
                    return new Drawings.Color(219, 112, 147, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "PapayaWhip", {
                get: function () {
                    return new Drawings.Color(255, 239, 213, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "PeachPuff", {
                get: function () {
                    return new Drawings.Color(255, 218, 185, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Peru", {
                get: function () {
                    return new Drawings.Color(205, 133, 63, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Pink", {
                get: function () {
                    return new Drawings.Color(255, 192, 203, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Plum", {
                get: function () {
                    return new Drawings.Color(221, 160, 221, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "PowderBlue", {
                get: function () {
                    return new Drawings.Color(176, 224, 230, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Purple", {
                get: function () {
                    return new Drawings.Color(128, 0, 128, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Red", {
                get: function () {
                    return new Drawings.Color(255, 0, 0, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "RosyBrown", {
                get: function () {
                    return new Drawings.Color(188, 143, 143, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "RoyalBlue", {
                get: function () {
                    return new Drawings.Color(65, 105, 225, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "SaddleBrown", {
                get: function () {
                    return new Drawings.Color(139, 69, 19, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Salmon", {
                get: function () {
                    return new Drawings.Color(250, 128, 114, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "SandyBrown", {
                get: function () {
                    return new Drawings.Color(244, 164, 96, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "SeaGreen", {
                get: function () {
                    return new Drawings.Color(46, 139, 87, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "SeaShell", {
                get: function () {
                    return new Drawings.Color(255, 245, 238, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Sienna", {
                get: function () {
                    return new Drawings.Color(160, 82, 45, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Silver", {
                get: function () {
                    return new Drawings.Color(192, 192, 192, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "SkyBlue", {
                get: function () {
                    return new Drawings.Color(135, 206, 235, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "SlateBlue", {
                get: function () {
                    return new Drawings.Color(106, 90, 205, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "SlateGray", {
                get: function () {
                    return new Drawings.Color(112, 128, 144, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Snow", {
                get: function () {
                    return new Drawings.Color(255, 250, 250, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "SpringGreen", {
                get: function () {
                    return new Drawings.Color(0, 255, 127, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "SteelBlue", {
                get: function () {
                    return new Drawings.Color(70, 130, 180, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Tan", {
                get: function () {
                    return new Drawings.Color(210, 180, 140, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Teal", {
                get: function () {
                    return new Drawings.Color(0, 128, 128, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Thistle", {
                get: function () {
                    return new Drawings.Color(216, 191, 216, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Tomato", {
                get: function () {
                    return new Drawings.Color(255, 99, 71, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Transparent", {
                get: function () {
                    return new Drawings.Color(255, 255, 255, 0);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Turquoise", {
                get: function () {
                    return new Drawings.Color(64, 224, 208, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Violet", {
                get: function () {
                    return new Drawings.Color(238, 130, 238, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Wheat", {
                get: function () {
                    return new Drawings.Color(245, 222, 179, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "White", {
                get: function () {
                    return new Drawings.Color(255, 255, 255, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "WhiteSmoke", {
                get: function () {
                    return new Drawings.Color(245, 245, 245, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Yellow", {
                get: function () {
                    return new Drawings.Color(255, 255, 0, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "YellowGreen", {
                get: function () {
                    return new Drawings.Color(154, 205, 50, 1);
                },
                enumerable: true,
                configurable: true
            });
            return Colors;
        })();
        Drawings.Colors = Colors;        
    })(CodeCompass.Drawings || (CodeCompass.Drawings = {}));
    var Drawings = CodeCompass.Drawings;
})(CodeCompass || (CodeCompass = {}));
//@ sourceMappingURL=Drawings.js.map
