///<reference path="../Scripts/raphael.d.ts" />
///<reference path="../Scripts/jquery-1.8.3.d.ts" />
///<reference path="../Scripts/linq.d.ts" />
///<reference path="Common.ts" />
///<reference path="Drawings.ts" />

module CodeCompass.Drawings.Raphael2D {
    //Vszont reprezentl osztly
    export class Canvas implements CodeCompass.Drawings.ICanvas {
        private _paper: RaphaelPaper;

        private _id: string;
        private _className: string;

        //visszatr az azonostjval
        get id(): string { return this._id; }
        //visszatr a class attribtummal
        get className(): string { return this._className; }

        //belltja az azonostt
        set id(value: string) {
            this._id = value;
            $("#" + value).attr("id", value);
        }

        //belltja az class attribtumt
        set className(value: string) {
            this._className = value;
            $("." + value).attr("class", value);
        }

        get paper(): RaphaelPaper { return this._paper; }

        constructor(element: string, public width: number = 1000, public height: number = width * 9 / 16) {
            Ensure.defined(element, "element: string (Canvas)")
                  .defined(width, "width: number (Canvas)")
                  .defined(height, "height: number (Canvas)");

            this._id = element;
            this._paper = Raphael(element, width, height);
        }
    }

    //Vonaltpus reprezentl osztly
    export class Stroke implements IStroke {
        private _color: Color;
        private _size: number;
        private _linecap: LineCap;
        private _linejoin: LineJoin;
        private _miterlimit: number;

        private _element: RaphaelElement;

        // visszatr 
        get element() { return this._element; }
        set element(value: RaphaelElement) {
            Ensure.defined(value, "element: RaphaelElement (Stroke)");
            this._element = value;
            this.refresh();
        }

        //a vonaltpus frisstse
        private refresh() {
            if (Ensure.hasValue(this.element)) {
                this.element.attr({
                    "stroke": Ensure.hasValue(this.color) ? this.color.toCSS() : "black",
                    "stroke-width": this.size,
                    "stroke-linecap": this.linecap,
                    "stroke-linejoin": this.linejoin,
                    "stroke-miterlimit": this.miterlimit
                });
            }
        }

        //vonal szne
        get color(): Color { return this._color; }
        //vonalvastagsggal tr vissza
        get size(): number { return this._size; }
        //vonal vgi szimblummal tr vissza
        get linecap(): LineCap { return this._linecap; }
        //vonal trsnek mdjval tr vissza
        get linejoin(): LineJoin { return this._linejoin; }
        get miterlimit(): number { return this._miterlimit; }

        //a vonal sznt lehet belltani vele
        set color(value: Color) {
            if (Ensure.hasValue(value)) {
                this._color = value;
                if (Ensure.hasValue(this.element))
                    this.element.attr({ "stroke": this.color.toCSS() });
            }
        }

        //vonal nagysgt lehet belltani
        set size(value: number) {
            if (Ensure.hasValue(value)) {
                this._size = value;
                if (Ensure.hasValue(this.element))
                    this.element.attr({ "stroke-width": this.size });
            }
        }

        //a vonalvgi szimblumot lehet vele belltani
        set linecap(value: LineCap) {
            if (Ensure.hasValue(value)) {
                this._linecap = value;
                if (Ensure.hasValue(this.element))
                    this.element.attr({ "stroke-linecap": this.linecap });
            }
        }

        //a vonal trsi stlust lehet belltani
        set linejoin(value: LineJoin) {
            if (Ensure.hasValue(value)) {
                this._linejoin = value;
                if (Ensure.hasValue(this.element))
                    this.element.attr({ "stroke-linejoin": this.linejoin });
            }
        }

        set miterlimit(value: number) {
            if (Ensure.hasValue(value)) {
                this._miterlimit = value;
                if (Ensure.hasValue(this.element))
                    this.element.attr({ "stroke-miterlimit": this.miterlimit });
            }
        }

        constructor(element: RaphaelElement) {
            this.color = Colors.Black;
            this.miterlimit = 0;
            this.size = 1;
            this.linecap = LineCap.butt;
            this.linejoin = LineJoin.bevel;
            this.element = element;
        }
    }
}

module CodeCompass.Drawings.Raphael2D.Utilities {

    //Skbeli ponton vgezhet mveleteket reprezentl lmodul
    export module Point2D {
        //SVG ttal visszatr fggvny, aminek egy pontokbl ll tmbt kell tadni
        export function getPath(...points: IPoint2D[]): string {
            return getPathFromArray(points);
        }

        //egy SVG ttal tr vissza
        export function getPathFromArray(points: IPoint2D[]): string {
            var result = "M";

            return Let(result).be(value => {
                var output = [];

                for (var index in points) {
                    output.push(points[index].x + " " + points[index].y);
                }
                value += output.join("L");
                return value;
            }).value;
        }

        export function arrange(container: ISize, points: IPoint2D[]): IPoint2D[] {
            Ensure.defined(container)
                  .defined(points);
            var list: IPoint2D[] = Enumerable.From(points)
                                             .Select((point: IPoint2D, index) => {

                                                 var absolute = true;
                                                 Safe(point).tryGet("isAbsolute", value => {
                                                     absolute = value;
                                                 });

                                                 var result: IPoint2D = {
                                                     x: absolute ? point.x : point.x * container.width,
                                                     y: absolute ? point.y : point.y * container.height
                                                 };

                                                 Safe(point).tryGet("horizontalAlign", value => {
                                                     switch (value) {
                                                         case HorizontalAlign.left:
                                                             result.x = point.x * container.width;
                                                             break;
                                                         case HorizontalAlign.center:
                                                             result.x = absolute ? point.x + container.width / 2.0 : (0.5 - point.x) * container.width;
                                                             break;
                                                         case HorizontalAlign.right:
                                                             result.x = absolute ? point.x + container.width : (1.0 - point.x) * container.width;
                                                             break;
                                                     }
                                                 }).tryGet("verticalAlign", value => {
                                                     switch (value) {
                                                         case VerticalAlign.top:
                                                             result.y = point.y * container.height;
                                                             break;
                                                         case VerticalAlign.center:
                                                             result.y = absolute ? point.y + container.height / 2.0 : (0.5 - point.y) * container.height;
                                                             break;
                                                         case VerticalAlign.bottom:
                                                             result.y = absolute ? point.y + container.height : (1.0 - point.y) * container.height;
                                                             break;
                                                     }
                                                 });

                                                 return new Drawings.Point2D(result.x, result.y);
                                             })
                                             .ToArray();

            return list;
        }

        //pontot mret objektumm konvertl segdfggvny
        export function toSize(value: IPoint2D): ISize {
            return { width: value.x, height: value.y };
        }
    }

    //ISize interfszbl leszrmazott elemekhez segdfggvnyeket tartalmaz modul
    export module Size {
        //ISize tpus objektumot pontt alakt fggvny
        export function toPoint(value: ISize): Drawings.Point2D {
            return Drawings.Point2D.create(value.width, value.height);
        }

        export function arrange(container: ISize, sizes: ISize[]) {
            var list = Enumerable.From(sizes)
                                 .Select((size: ISize, index: number) => {
                                     var result: ISize;

                                     if (Safe(size).has("isAbsolute"))
                                         result = new Drawings.Size(container.width * size.width, container.height * size.height);
                                     else
                                         result = new Drawings.Size(size.width, size.height);

                                     return result;
                                 })
                                 .ToArray();

            return <ISize[]>list;
        }
    }
}

//Primitveket trol modul
module CodeCompass.Drawings.Raphael2D.Primitives {

    //Raphael alap grafikai objektumot reprezentl osztly
    export class GraphicalObject implements CodeCompass.Drawings.Primitives.IGraphicalObject, IDisposable {
        private _element: RaphaelElement;
        private _location: IPoint2D;
        private _visible: bool;
        private _size: ISize;
        private _isDisposed: bool = false;

        get isDisposed() { return this._isDisposed; }

        //Visszatr az objektum Raphel-beli mretvel
        get size() { return this._size; }
        //Raphael-beli mretet lehet vele belltani
        set size(value: ISize) {
            Ensure.defined(value);
            this._size = value;
            if (Ensure.isDefined(this._element)) {
                this._element.attr({ "width": value.width, "height": value.height, "rx": value.width / 2.0, "ry": value.height / 2.0 });
            }
        }
        //Igazzal tr vissza, ha lthat az objektum
        get visible() { return this._visible; }

        //Az objektum lthatsgt lehet vele belltani
        set visible(value: bool) {
            Ensure.defined(value, "value: bool (GraphicalObject)");
            this._visible = value;

            if (value) this.element.show();
            else this.element.hide();
        }

        //Visszatr a Raphael-beli elemmel
        get element() { return this._element; }

        //Raphael-beli elemet lehet vele belltani
        set element(value: RaphaelElement) {
            this._element = value;
        }

        //Visszatr az elem helyzetvel
        get location() { return this._location; }
        //Az objektum pozicijt lehet vele belltani
        set location(value: IPoint2D) {
            Ensure.defined(value, "value: IPoint2D (GraphicalObject)");
            this._location = value;
            this._element.attr({ "x": value.x, "y": value.y, "cx": value.x, "cy": value.y });
        }

        public dispose() {
            if (!this.isDisposed && Ensure.isDefined(this.element)) {
                this.element.remove();
                this._isDisposed = true;
            }
            return this;
        }

        //Konstruktor
        constructor(public canvas: ICanvas, location: IPoint2D, size: ISize) {
            this._location = location;
            this._visible = true;
            this._size = size;
        }
    }

    //Geometriai alakzatot reprezentl osztly
    export class Shape extends GraphicalObject implements CodeCompass.Drawings.Primitives.IShape {
        private _element: RaphaelElement;
        private _border: Stroke;
        private _fill: Color;

        //Az alakzat kitltsi sznt lehet vele belltani
        set fill(value: Color) {
            if (Ensure.hasValue(value)) {
                this._fill = value;
                if (Ensure.hasValue(this.element))
                    this.element.attr({ fill: value.toCSS() });
            }
        }

        //Visszatr az alakzat kitltsi sznvel
        get fill() { return this._fill; }

        //Visszatr az alakzat krvonalval
        get border() { return this._border; }
        get element() { return this._element; }

        set element(value: RaphaelElement) {
            Ensure.defined(value, "value: RaphaelElement (Shape)");

            this._element = value;
            if (!Ensure.hasValue(this.border))
                this._border = new Stroke(value);
            else
                this.border.element = value;

            this.element.attr({ fill: this.fill.toCSS() });
        }

        constructor(canvas: Canvas, location: IPoint2D = Point2D.Origo, size?: ISize = Size.Zero) {
            super(canvas, location, size);
            Ensure.defined(canvas, "canvas: Canvas (Shape)")
                  .defined(location, "location: IPoint (Shape)")
                  .defined(size, "size: ISize (Shape)");

            this._border = null;
            this.fill = Colors.Blue;
        }
    }

    //Tglalapot reprezentl osztly
    export class Rectange extends Shape
                          implements CodeCompass.Drawings.Primitives.IRectange {

        private init() {
            this.element = (<Canvas>this.canvas).paper.rect(this.location.x, this.location.y, this.size.width, this.size.height);
        }

        constructor(canvas: Canvas, location: IPoint2D = Point2D.Origo, size: ISize = Size.Zero) {
            super(canvas, location, size);
            this.init();
        }
    }

    //Ellipszist reprezentl osztly
    export class Ellipse extends Shape
                         implements CodeCompass.Drawings.Primitives.IEllipse {
        private init() {
            this.element = (<Canvas>this.canvas).paper.ellipse(this.location.x, this.location.y, this.size.width / 2.0, this.size.height / 2.0);
        }

        constructor(canvas: Canvas, location: IPoint2D = Point2D.Origo, size: ISize = Size.Zero) {
            super(canvas, location, size);
            this.init();
        }
    }

    //Polygon-t reprezentl osztly
    export class Polygon extends Shape
                         implements CodeCompass.Drawings.Primitives.IPolygon {
        private _location: IPoint2D;
        private _closed = false;
        private _size: ISize;

        public get isClosed() { return this._closed; }

        get size() { return this._size; }
        set size(value: ISize) {
            Ensure.defined(value);

            this._size = value;

            if (Ensure.isDefined(this.element))
                this.init();
        }

        get location() { return this._location; }

        set location(value: IPoint2D) {
            Ensure.defined(value);
            this._location = value;
            this.element.translate(value.x, value.y);
        }

        private init() {
            if (this.points.length === 0) return;

            /*var points = Enumerable.From(this.points)
                                   .Select((point: IPoint2D, index) => {

                                       var absolute = false;
                                       Safe(point).tryGet("isAbsolute", value => {
                                           absolute = value;
                                       });

                                       var result: IPoint2D = new RelativePoint2D(point.x, point.y, absolute);

                                       Safe(point).tryGet("horizontalAlign", value => {
                                           var x: number;

                                           switch (value) {
                                               case HorizontalAlign.left:
                                                   x = point.x;
                                                   break;
                                               case HorizontalAlign.center:
                                                   x = absolute ? this.size.width / 2.0 : 0.5 - point.x;
                                                   break;
                                               case HorizontalAlign.right:
                                                   x = absolute ? this.size.width : 1.0 - point.x;
                                                   break;
                                           }
                                           result = new RelativePoint2D(x, result.y, absolute);
                                       }).tryGet("verticalAlign", value => {
                                           var y: number;
                                           switch (value) {
                                               case VerticalAlign.top:
                                                   y = point.y;
                                                   break;
                                               case VerticalAlign.center:
                                                   y = absolute ? this.size.height / 2.0 : 0.5 - point.y;
                                                   break;
                                               case VerticalAlign.bottom:
                                                   y = absolute ? this.size.height : 1.0 - point.y;
                                                   break;
                                           }
                                           result = new RelativePoint2D(result.x, y, absolute);
                                       });

                                       if (absolute)
                                           return result;

                                       return new Point2D(result.x * this.size.width, result.y * this.size.height);
                                   })
                                   .Select((point: IPoint2D, index: number) => {
                                       var result = "L " + point.x + "," + point.y;
                                       if (index === 0)
                                           result = "M " + [point.x, point.y].join(",") + result;
                                       return result;
                                   })
                                   .ToArray();*/

            var points = Enumerable.From(Utilities.Point2D.arrange(this.size, this.points))
                                   .Select((point: IPoint2D, index: number) => {
                                       var result = "L " + point.x + "," + point.y;
                                       if (index === 0)
                                           result = "M " + [point.x, point.y].join(",") + result;
                                       return result;
                                   })
                                   .ToArray();


            var path = points.join(" ");

            if (this.isClosed)
                path += "Z";

            this.element = (<Canvas>this.canvas).paper.path(path).translate(this.location.x, this.location.y);
        }

        constructor(canvas: Canvas, public points: IPoint2D[], closed: bool = true, location: IPoint2D = Point2D.Origo, size: ISize = Size.Zero) {
            super(canvas, location, size);
            this._closed = closed;
            this.init();
        }
    }

    //Rombuszt reprezentl osztly
    export class Rhombus extends Polygon {
        constructor(canvas: Canvas, location: IPoint2D = Point2D.Origo, size: ISize = new Size(100, 100)) {
            super(canvas, [{ x: 0, y: 0, verticalAlign: VerticalAlign.top, horizontalAlign: HorizontalAlign.center, isAbsolute: true },
                           { x: 0, y: 0, verticalAlign: VerticalAlign.center, horizontalAlign: HorizontalAlign.right, isAbsolute: true },
                           { x: 0, y: 0, verticalAlign: VerticalAlign.bottom, horizontalAlign: HorizontalAlign.center, isAbsolute: true },
                           { x: 0, y: 0, verticalAlign: VerticalAlign.center, horizontalAlign: HorizontalAlign.left, isAbsolute: true }],
                          true, location, size);
        }
    }

    //Nyilat reprezentl osztly
    export class Arrow extends Polygon {
        constructor(canvas: Canvas, location: IPoint2D = Point2D.Origo, size: ISize = new Size(100, 100)) {
            super(canvas, [{ x: 0, y: 0.3, isAbsolute: false },
                           { x: 0.6, y: 0.3, isAbsolute: false },
                           { x: 0.6, y: 0, isAbsolute: false },
                           { x: 1, y: 0.5, isAbsolute: false },
                           { x: 0.6, y: 1, isAbsolute: false },
                           { x: 0.6, y: 0.7, isAbsolute: false },
                           { x: 0, y: 0.7, isAbsolute: false }],
                          true, location, size);
        }
    }

    //Szveget reprezentl osztly
    export class Text extends GraphicalObject
                      implements Drawings.Primitives.IText {
        private _horizontalAlign: HorizontalAlign;
        private _location: IPoint2D;
        private _foreground: Color;
        private _background: Color;
        private _fontSize: number;
        private _fontFamily: string;
        private _text: string;

        //Visszatr a szveg mretvel
        get size() {
            if (Ensure.isDefined(this.element)) {
                var boundingBox = this.element.getBBox();
                return new Drawings.Size(boundingBox.width, boundingBox.height);
            }
            return Drawings.Size.Zero;
        }

        //Visszatr a szveg tartalmval
        get text() { return this._text; }
        //A szveg tartalmt lehet vele belltani
        set text(value: string) {
            Ensure.defined(this.element).defined(value);
            this._text = value;
            this.element.attr("text", value);
        }
        //Visszatr a szveg mretvel
        get fontSize() { return this._fontSize; }
        //A szveg mrett lehet vel belltani
        set fontSize(value: number) {
            Ensure.defined(value).defined(this.element);
            this._fontSize = value;
            this.element.attr({ "font-size": value });
        }

        //A szveghez tartoz font csaldot lehet lekrdezni
        get fontFamily() { return this._fontFamily; }
        //A szveghez tartoz font csaldot lehet belltani
        set fontFamily(value: string) {
            Ensure.notBlank(value).defined(this.element);
            this._fontFamily = value;
            this.element.attr({ "font-family": value });
        }

        //A szveg sznt lehet lekrdezni
        get foreground() { return this._foreground; }
        //A szveg sznt lehet vele belltani
        set foreground(value: Color) {
            Ensure.defined(this.element).defined(value);
            this._foreground = value;
            this.element.attr({ "fill": value.toCSS() });
        }

        //A szveg vzszintes elhelyezkedst lehet belltani, illetve lekrdezni
        get horizontalAlign() { return this._horizontalAlign; }
        set horizontalAlign(value: HorizontalAlign) {
            Ensure.defined(value);
            var position: string;

            switch (value) {
                case HorizontalAlign.left:
                    position = "start";
                    break;
                case HorizontalAlign.center:
                    position = "middle";
                    break;
                case HorizontalAlign.right:
                    position = "end";
                    break;
            }

            this.element.attr({ "text-anchor": position });
            this._horizontalAlign = value;
        }

        //A szveg pozicijt lehet lekrdezni s belltani
        get location(): IPoint2D { return this._location; }
        set location(value: IPoint2D) {
            Ensure.defined(value);
            this._location = new Point2D(value.x, value.y);
            Ensure.defined(this.element);
            this.element.attr("x", value.x);
            this.element.attr({ "y": value.y + this.fontSize / 2.0 });
        }

        private init() {
            this.element = (<Canvas>this.canvas).paper.text(this.location.x, this.location.y, this.text);
            this.element.attr({
                "font-family": this.fontFamily,
                "font-size": this.fontSize,
                "text-anchor": "start",
                "fill": this.foreground.toCSS(),
                "y": this.location.y + this.fontSize / 2.0
            });

            var boundingBox = this.element.getBBox();
            this.size = new Size(boundingBox.width, boundingBox.height);
        }

        constructor(canvas: Canvas, text: string, location: IPoint2D = Point2D.Origo, fontSize: number = 10, fontFamily: string = "Arial", foreground: Color = Colors.Black, background: Color = Colors.Transparent) {
            super(canvas, location, new Size(0, 0));

            this._text = text;
            this._fontSize = fontSize;
            this._fontFamily = fontFamily;
            this._foreground = foreground;
            this._background = background;

            this.init();
        }
    }

    //Logikai halmazt reprezentl osztly, amivel csoportosthatjuk a Raphael objektumokat
    export class Group extends GraphicalObject implements Drawings.Primitives.IGroup, CodeCompass.IDisposable {
        private _children: GraphicalObject[];
        private _set: RaphaelSet;
        private _location: IPoint2D;
        private _visible: bool;
        private _size: Size;
        private _isDisposed: bool = false;

        get isDisposed() { return this._isDisposed; }

        get size() { return this._size; }
        set size(value: Size) {
            Ensure.defined(value);
            this._size = value;
            this.update();
        }

        get visible() { return this._visible; }
        set visible(value: bool) {
            Ensure.defined(value);
            this._visible = value;
            for (var child in this.children) {
                this.children[child].visible = value;
            }
        }

        get location() { return this._location; }

        set location(value: IPoint2D) {
            Ensure.defined(value);
            this._location = value;

            for (var child in this.children) {
                var item: Drawings.Primitives.IGraphicalObject = this.children[child];
                var location = item.location;
                Ensure.defined(location, "location");
                item.location = location.add(value);
            }
        }

        get children() { return this._children; }

        public dispose() {
            if (!this.isDisposed && Ensure.isDefined(this._children)) {
                this._children.forEach(child => child.dispose());
                this._isDisposed = true;
            }

            return this;
        }

        private update(list: GraphicalObject[] = this._children) {
            if (Ensure.isDefined(list)) {
                list.forEach(child => {
                    child.location = Enumerable.From(Utilities.Point2D.arrange(this.size, [child.location])).First();
                    child.size = Enumerable.From(Utilities.Size.arrange(this.size, [child.size])).First();

                    if (Ensure.hasValue(child) && Ensure.hasValue(child.element))
                        child.element.toFront();
                });
            }
        }

        ///TODO: remove Raphael objects from canvas
        public clear(): Group {
            return this;
        }

        public ungroup(): Group {
            Let(this.location).be(value => {
                var location = value;
                this.location = Point2D.Origo;
                this._children = [];
                this._set.clear();
                this.location = location;
            });
            return this;
        }

        public add(...children: GraphicalObject[]): Group {
            Ensure.defined(children, "child: GraphicalObject (Group)");

            children.forEach(value => {
                this._children.push(value);
                this._set.push(value.element);
            });

            this.update(children);

            return this;
        }

        private init() {
            this._children = [];
            this._set = (<Drawings.Raphael2D.Canvas>this.canvas).paper.set();
        }

        constructor(public canvas: ICanvas, location: IPoint2D = Point2D.Origo, size: Size = new Drawings.Size(0, 0)) {
            super(canvas, location, size);
            this._size = size;
            this.init();
        }
    }

    //Skbeli nyilat reprezentl osztly
    export class Line2D extends GraphicalObject implements Drawings.Primitives.ILine2D {
        private _stroke: Stroke;
        private _start: Point2D;
        private _end: Point2D;
        private _arrowEnd: ArrowEnd;

        get arrowEnd() { return this._arrowEnd; }
        set arrowEnd(value: ArrowEnd) {
            Ensure.defined(value)
                  .defined(this.element, "this.element: RaphaelElement (Line2D)");
            this._arrowEnd = value;
            switch (value) {
                case ArrowEnd.block:
                    this.element.attr("arrow-end", "block");
                    break;
                case ArrowEnd.classic:
                    this.element.attr("arrow-end", "classic");
                    break;
                case ArrowEnd.diamond:
                    this.element.attr("arrow-end", "diamond");
                    break;
                case ArrowEnd.none:
                    this.element.attr("arrow-end", "none");
                    break;
                case ArrowEnd.open:
                    this.element.attr("arrow-end", "open");
                    break;
                case ArrowEnd.oval:
                    this.element.attr("arrow-end", "oval");
                    break;
            }
        }

        get start() { return this._start; }
        set start(value: Point2D) {
            Ensure.defined(value)
                  .defined(this.element);
            this._start = value;
            this.location = value;
            this.element.attr("path", Utilities.Point2D.getPath(this.start, this.end));
        }

        get end() { return this._end; }
        set end(value: Point2D) {
            Ensure.defined(value)
                  .defined(this.element);
            this._end = value;
            this.element.attr("path", Utilities.Point2D.getPath(this.start, this.end));
        }

        get stroke() { return this._stroke; }

        private init() {
            this.element = (<Canvas>this.canvas).paper.path(Utilities.Point2D.getPath(this.start, this.end));
            this._stroke = new Stroke(this.element);
            this.arrowEnd = ArrowEnd.none;
            this.element.attr("stroke-width", 4);
        }

        constructor(canvas: Canvas, start: Point2D, end: Point2D) {
            super(canvas, start, new Size(Math.abs(end.add(start.negate()).x),
                                          Math.abs(end.add(start.negate()).y)));
            this._start = start;
            this._end = end;

            this.init();
        }
    }

    //TODO: support relative Points too!!!
    //Tbbirny (trt) nyilat reprezentl osztly
    export class MultiLine2D extends Line2D {
        private _points: Point2D[];

        public getPoints() { return Enumerable.From(this._points); }

        public setPoints(value: Point2D[]) {
            Ensure.defined(value)
                  .defined(this.element);
            this._points = value;
            this.element.attr("path", Utilities.Point2D.getPathFromArray(value));
        }

        constructor(canvas: Canvas, start: Point2D = Point2D.Origo, end: Point2D = Point2D.Origo) {
            super(canvas, start, end);
            this.start = start;
            this.end = end;
        }
    }
}