var __extends = this.__extends || function (d, b) {
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
var CodeCompass;
(function (CodeCompass) {
    (function (Floyd) {
        (function (Diagram) {
            var drawings = CodeCompass.Drawings;
            var Point2D = drawings.Point2D;
            var Size = drawings.Sizedfhdfj;
            var primitives = CodeCompass.Drawings.Raphael2D.Primitives;
            var utilities = CodeCompass.Drawings.Raphael2D.Utilities;
            var Colors = drawings.Colors;
            var toPoint = utilities.Size.toPoint;
            var toSize = utilities.Point2D.toSize;
            var getClass = Common.getClass;
            var FloydGraph = (function () {
                function FloydGraph() { }
                FloydGraph.prototype.getNodes = function (location, size, func) {
                    Ensure.defined(location).defined(size).defined(func);
                    for(var i = Math.max(location.x, 0); i < Math.min(location.x + size.width, this._graphSize.width); ++i) {
                        for(var j = Math.max(location.y, 0); j < Math.min(location.y + size.height, this._graphSize.height); ++j) {
                            this.nodes[j][i] = func(this.nodes[j][i], new CodeCompass.Drawings.Point2D(i, j));
                        }
                    }
                    return this;
                };
                FloydGraph.prototype.initNodes = function (size) {
                    Ensure.defined(size).defined(size.width).defined(size.height);
                    this._graphSize = size;
                    this.nodes = [];
                    for(var i = 0; i < size.width; ++i) {
                        this.nodes.push([]);
                        for(var j = 0; j < size.height; ++j) {
                            this.nodes[this.nodes.length - 1].push(1);
                        }
                    }
                    return this;
                };
                FloydGraph.prototype.planRoute = function (start, end) {
                    var _this = this;
                    Ensure.defined(start).defined(end);
                    var distance = [];
                    var parent = [];
                    var isChanged = true;
                    for(var i = 0; i < this._graphSize.width; ++i) {
                        distance.push([]);
                        parent.push([]);
                        for(var j = 0; j < this._graphSize.height; ++j) {
                            distance[i].push(Number.POSITIVE_INFINITY);
                            parent[i].push(null);
                        }
                    }
                    distance[start.y][start.x] = 0;
                    var checkEdge = function (u, v) {
                        if(v.x < 0 || v.x >= _this._graphSize.width || v.y < 0 || v.y >= _this._graphSize.height) {
                            return;
                        }
                        if(distance[v.y][v.x] > distance[u.y][u.x] + _this.nodes[u.y][u.x]) {
                            distance[v.y][v.x] = distance[u.y][u.x] + _this.nodes[u.y][u.x];
                            parent[v.y][v.x] = {
                                x: u.x,
                                y: u.y
                            };
                            isChanged = true;
                        }
                    };
                    for(var k = 0; k <= this._graphSize.width * this._graphSize.height - 1 && isChanged; ++k) {
                        isChanged = false;
                        for(var i = 0; i < this._graphSize.width; ++i) {
                            for(var j = 0; j < this._graphSize.height; ++j) {
                                [
                                    {
                                        x: i - 1,
                                        y: j
                                    }, 
                                    {
                                        x: i + 1,
                                        y: j
                                    }, 
                                    {
                                        x: i,
                                        y: j - 1
                                    }, 
                                    {
                                        x: i,
                                        y: j + 1
                                    }
                                ].forEach(function (value) {
                                    var u = {
                                        x: i,
                                        y: j
                                    };
                                    checkEdge(u, value);
                                });
                            }
                        }
                    }
                    var route = [
                        end
                    ];
                    var p = parent[end.y][end.x];
                    this.addObject(end, new Size(1, 1), 100);
                    while(p !== null) {
                        this.addObject(p, new Size(1, 1), 100);
                        route.push(p);
                        p = parent[p.y][p.x];
                    }
                    return route.reverse();
                };
                FloydGraph.prototype.removeObject = function (location, size) {
                    this.getNodes(location, size, function (node) {
                        return 1;
                    });
                    return this;
                };
                FloydGraph.prototype.addObject = function (location, size, weight) {
                    if (typeof weight === "undefined") { weight = 10000; }
                    this.getNodes(location, size, function (node) {
                        return weight;
                    });
                    return this;
                };
                return FloydGraph;
            })();
            Diagram.FloydGraph = FloydGraph;            
            var FloydDocument = (function () {
                function FloydDocument(elementName) {
                    if (typeof elementName === "undefined") { elementName = "canvas"; }
                    this._isDisposed = false;
                    Ensure.notBlank(elementName, "elementName: string (FloydDocument)");
                    this._canvasSize = new Size(1000, 1000);
                    this._canvas = new CodeCompass.Drawings.Raphael2D.Canvas(elementName, this._canvasSize.width, this._canvasSize.height);
                    this._size = new Size(50, 50);
                    this._graph = new FloydGraph();
                    this._graph.initNodes(this._size);
                    this._children = [];
                }
                Object.defineProperty(FloydDocument.prototype, "isDisposed", {
                    get: function () {
                        return this._isDisposed;
                    },
                    enumerable: true,
                    configurable: true
                });
                Object.defineProperty(FloydDocument.prototype, "children", {
                    get: function () {
                        return this._children;
                    },
                    enumerable: true,
                    configurable: true
                });
                Object.defineProperty(FloydDocument.prototype, "start", {
                    get: function () {
                        return this._start;
                    },
                    enumerable: true,
                    configurable: true
                });
                Object.defineProperty(FloydDocument.prototype, "end", {
                    get: function () {
                        return this._end;
                    },
                    enumerable: true,
                    configurable: true
                });
                Object.defineProperty(FloydDocument.prototype, "size", {
                    get: function () {
                        return this._size;
                    },
                    enumerable: true,
                    configurable: true
                });
                FloydDocument.prototype.getCanvas = function () {
                    return this._canvas;
                };
                FloydDocument.prototype.fromPoint = function (location) {
                    return Point2D.create(Math.floor(location.x / (this._canvasSize.width / this.size.width)), Math.floor(location.y / (this._canvasSize.height / this.size.height)));
                };
                FloydDocument.prototype.getPoint = function (location) {
                    return Point2D.create(location.x * (this._canvasSize.width / this.size.width), location.y * (this._canvasSize.height / this.size.height));
                };
                FloydDocument.prototype.getSize = function (size) {
                    return new Size(size.width * (this._canvasSize.width / this.size.width), size.height * (this._canvasSize.height / this.size.height));
                };
                FloydDocument.prototype.clear = function () {
                    this._children = [];
                    return this;
                };
                FloydDocument.prototype.connectChildren = function (element) {
                    var _this = this;
                    Ensure.defined(element);
                    if(getClass(element) === FloydCondition) {
                        var condition = element;
                        this.connectElement(condition, condition.trueCase, 0);
                        this.connectElement(condition, condition.falseCase, 1);
                        this.connectChildren(condition.trueCase).connectChildren(condition.falseCase);
                    } else if(getClass(element) === FloydLoop) {
                        var loop = element;
                        if(Ensure.hasValue(loop.core)) {
                            this.connectChildren(loop.core);
                            this.connectElement(loop, loop.core, 1);
                            this.connectElement(loop.core, loop, 0, 1);
                        }
                    } else if(getClass(element) === FloydBlock) {
                        var block = element;
                        block.children.forEach(function (child) {
                            Ensure.defined(child);
                            _this.connectChildren(child);
                            if(Ensure.hasValue(child.next)) {
                                _this.connectElement(child, child.next);
                            }
                        });
                    }
                    return this;
                };
                FloydDocument.prototype.connectElement = function (start, end, startIndex, endIndex) {
                    if (typeof startIndex === "undefined") { startIndex = 0; }
                    if (typeof endIndex === "undefined") { endIndex = 0; }
                    var _this = this;
                    Ensure.defined(start).defined(end);
                    if(getClass(start) === FloydCondition && getClass(end) === FloydConditionEndElement) {
                        var condition = start;
                        this.connectElement(condition.trueCase, end);
                        this.connectElement(condition.falseCase, end);
                    } else if(getClass(start) === FloydBlock) {
                        var block = start;
                        this.connectElement(Enumerable.From(block.children).Last(), end, startIndex, endIndex);
                    } else if(getClass(end) === FloydBlock) {
                        var block = end;
                        this.connectElement(start, Enumerable.From(block.children).First(), startIndex, endIndex);
                    } else {
                        var startPoint = start.outputPoints[startIndex];
                        var endPoint = end.inputPoints[endIndex];
                        startPoint = startPoint.add(start.element.location);
                        endPoint = endPoint.add(end.element.location);
                        var route = this._graph.planRoute(this.fromPoint(startPoint), this.fromPoint(endPoint));
                        var arrow = new CodeCompass.Drawings.Raphael2D.Primitives.MultiLine2D(this._canvas, startPoint, endPoint);
                        arrow.arrowEnd = CodeCompass.Drawings.ArrowEnd.block;
                        arrow.stroke.size = 3;
                        this._arrows.push(arrow);
                        var points = Enumerable.From(route).TakeExceptLast().Select(function (value) {
                            return _this.getPoint(new CodeCompass.Drawings.Point2D(value.x, value.y));
                        }).Select(function (value) {
                            return CodeCompass.Drawings.Point2D.create(value.x + _this._canvasSize.width / _this.size.width / 2.0, value.y + _this._canvasSize.height / _this.size.height / 2.0);
                        }).ToArray();
                        points.unshift(startPoint);
                        points.push(endPoint);
                        arrow.setPoints(points);
                    }
                    return this;
                };
                FloydDocument.prototype.arrangeElement = function (element, location) {
                    if (typeof location === "undefined") { location = CodeCompass.Drawings.Point2D.create(10, 1); }
                    var _this = this;
                    Ensure.defined(element);
                    var returnSize = new CodeCompass.Drawings.Size(0, 0);
                    if(getClass(element) === FloydCondition) {
                        var condition = element;
                        var trueCaseSize = this.arrangeElement(condition.trueCase, location.add(CodeCompass.Drawings.Point2D.create(0, condition.size.height + 2)));
                        var falseCaseSize = this.arrangeElement(condition.falseCase, location.add(CodeCompass.Drawings.Point2D.create(trueCaseSize.width + 2, condition.size.height + 2)));
                        condition.location = CodeCompass.Drawings.Point2D.create(location.x + Math.floor((trueCaseSize.width + falseCaseSize.width + 2) / 2.0 - condition.size.width / 2.0), location.y);
                        returnSize = new CodeCompass.Drawings.Size(trueCaseSize.width + falseCaseSize.width + 2, Math.max(trueCaseSize.height, falseCaseSize.height) + condition.size.height);
                    } else if(getClass(element) === FloydBlock) {
                        var block = element;
                        block.children.forEach(function (child) {
                            var size = _this.arrangeElement(child, location);
                            location = location.add(CodeCompass.Drawings.Point2D.create(0, size.height + 2));
                            returnSize = new CodeCompass.Drawings.Size(Math.max(returnSize.width, size.width), returnSize.height + size.height + 2);
                        });
                    } else if(getClass(element) === FloydLoop) {
                        var loop = element;
                        loop.location = location;
                        var size = this.arrangeElement(loop.core, location.add(CodeCompass.Drawings.Point2D.create(0, loop.size.height + 2)));
                        returnSize = new CodeCompass.Drawings.Size(Math.max(loop.size.width, size.width), loop.size.height + size.height);
                    } else {
                        element.location = location;
                        returnSize = element.size;
                    }
                    if(getClass(element) !== FloydBlock) {
                        this._graph.addObject(element.location, element.size);
                    }
                    return returnSize;
                };
                FloydDocument.prototype.arrangeLayout = function () {
                    this._graph = new FloydGraph();
                    this._graph.initNodes(this._size);
                    this._arrows = [];
                    var block = new FloydBlock(this, this._children);
                    this.arrangeElement(block);
                    this.connectChildren(block);
                    return this;
                };
                FloydDocument.prototype.init = function () {
                    Ensure.defined(this._children);
                    this._children.forEach(function (e) {
                        return e.init();
                    });
                };
                FloydDocument.prototype.dispose = function () {
                    if(!this._isDisposed && Ensure.isDefined(this._children)) {
                        this._children.forEach(function (e) {
                            return e.dispose();
                        });
                        this._isDisposed = true;
                    }
                };
                return FloydDocument;
            })();
            Diagram.FloydDocument = FloydDocument;            
            var FloydElement = (function () {
                function FloydElement(document, previous, next) {
                    if (typeof previous === "undefined") { previous = null; }
                    if (typeof next === "undefined") { next = null; }
                    this.previous = previous;
                    this.next = next;
                    Ensure.defined(document, "document: FloydDocument (FloydElement)");
                    this._document = document;
                    this._inputPoints = [];
                    this._outputPoints = [];
                    this._size = new CodeCompass.Drawings.Size(5, 3);
                    this._isDisposed = true;
                }
                Object.defineProperty(FloydElement.prototype, "element", {
                    get: function () {
                        return this._element;
                    },
                    enumerable: true,
                    configurable: true
                });
                Object.defineProperty(FloydElement.prototype, "inputPoints", {
                    get: function () {
                        return this._inputPoints;
                    },
                    enumerable: true,
                    configurable: true
                });
                Object.defineProperty(FloydElement.prototype, "outputPoints", {
                    get: function () {
                        return this._outputPoints;
                    },
                    enumerable: true,
                    configurable: true
                });
                Object.defineProperty(FloydElement.prototype, "size", {
                    get: function () {
                        return this._size;
                    },
                    enumerable: true,
                    configurable: true
                });
                Object.defineProperty(FloydElement.prototype, "location", {
                    get: function () {
                        return this._location;
                    },
                    set: function (value) {
                        Ensure.defined(value);
                        this._location = value;
                        this.element.location = this._document.getPoint(value);
                    },
                    enumerable: true,
                    configurable: true
                });
                FloydElement.prototype.init = function () {
                    this.dispose();
                    this._isDisposed = false;
                };
                FloydElement.prototype.dispose = function () {
                    if(Ensure.isDefined(this._element)) {
                        this._element.dispose();
                        this._isDisposed = true;
                    }
                };
                Object.defineProperty(FloydElement.prototype, "document", {
                    get: function () {
                        return this._document;
                    },
                    enumerable: true,
                    configurable: true
                });
                Object.defineProperty(FloydElement.prototype, "isDisposed", {
                    get: function () {
                        return this._isDisposed;
                    },
                    enumerable: true,
                    configurable: true
                });
                return FloydElement;
            })();
            Diagram.FloydElement = FloydElement;            
            var FloydStartElement = (function (_super) {
                __extends(FloydStartElement, _super);
                function FloydStartElement(document) {
                                _super.call(this, document);
                }
                FloydStartElement.prototype.init = function () {
                    _super.prototype.init.call(this);
                    var group = new CodeCompass.Drawings.Raphael2D.Primitives.Group(this.document.getCanvas(), Point2D.Origo, this.document.getSize(this.size));
                    var ellipse = new primitives.Ellipse(this.document.getCanvas(), this.document.getPoint(Point2D.create(2.5, 1.5)), new CodeCompass.Drawings.RelativeSize(1, 1));
                    ellipse.border.color = Colors.CornflowerBlue;
                    ellipse.fill = Colors.White;
                    ellipse.border.size = 3;
                    var text = new primitives.Text(this.document.getCanvas(), "start", Point2D.create(group.size.width / 2.0, group.size.height / 2.0 - 7.5), 15, "Verdana", Colors.DarkBlue);
                    text.horizontalAlign = CodeCompass.Drawings.HorizontalAlign.center;
                    this._element = group.add(ellipse, text);
                    this.inputPoints.push(CodeCompass.Drawings.Point2D.create(this.element.size.width / 2.0, 0));
                    this.outputPoints.push(CodeCompass.Drawings.Point2D.create(this.element.size.width / 2.0, this.element.size.height));
                };
                return FloydStartElement;
            })(FloydElement);
            Diagram.FloydStartElement = FloydStartElement;            
            var FloydConditionEndElement = (function (_super) {
                __extends(FloydConditionEndElement, _super);
                function FloydConditionEndElement(document) {
                                _super.call(this, document);
                }
                FloydConditionEndElement.prototype.init = function () {
                    _super.prototype.init.call(this);
                    var group = new CodeCompass.Drawings.Raphael2D.Primitives.Group(this.document.getCanvas(), Point2D.Origo, this.document.getSize(this.size));
                    var ellipse = new primitives.Ellipse(this.document.getCanvas(), Point2D.create(group.size.width / 2.0, group.size.height / 2.0 - 7.5), new CodeCompass.Drawings.RelativeSize(0.9, 0.9));
                    ellipse.border.color = Colors.CornflowerBlue;
                    ellipse.fill = Colors.White;
                    ellipse.border.size = 3;
                    var text = new primitives.Text(this.document.getCanvas(), "condition\nend", Point2D.create(group.size.width / 2.0, group.size.height / 2.0 - 15), 15, "Verdana", Colors.DarkBlue);
                    text.horizontalAlign = CodeCompass.Drawings.HorizontalAlign.center;
                    this._element = group.add(ellipse, text);
                    this.inputPoints.push(CodeCompass.Drawings.Point2D.create(this.element.size.width / 2.0, 0));
                    this.outputPoints.push(CodeCompass.Drawings.Point2D.create(this.element.size.width / 2.0, this.element.size.height - 7.5));
                };
                return FloydConditionEndElement;
            })(FloydElement);
            Diagram.FloydConditionEndElement = FloydConditionEndElement;            
            var FloydEndElement = (function (_super) {
                __extends(FloydEndElement, _super);
                function FloydEndElement(document) {
                                _super.call(this, document);
                }
                FloydEndElement.prototype.init = function () {
                    _super.prototype.init.call(this);
                    var group = new CodeCompass.Drawings.Raphael2D.Primitives.Group(this.document.getCanvas(), Point2D.Origo, this.document.getSize(this.size));
                    var ellipse = new primitives.Ellipse(this.document.getCanvas(), Point2D.create(group.size.width / 2.0, group.size.height / 2.0 - 7.5), new CodeCompass.Drawings.RelativeSize(0.9, 0.9));
                    ellipse.border.color = Colors.CornflowerBlue;
                    ellipse.fill = Colors.White;
                    ellipse.border.size = 3;
                    var text = new primitives.Text(this.document.getCanvas(), "end", this.document.getPoint(Point2D.create(2, 1)).add(Point2D.create(0, -7.5)), 15, "Verdana", Colors.DarkBlue);
                    text.horizontalAlign = CodeCompass.Drawings.HorizontalAlign.center;
                    this._element = group.add(ellipse, text);
                    this.inputPoints.push(CodeCompass.Drawings.Point2D.create(this.element.size.width / 2.0, 0));
                    this.outputPoints.push(CodeCompass.Drawings.Point2D.create(this.element.size.width / 2.0, this.element.size.height));
                };
                return FloydEndElement;
            })(FloydElement);
            Diagram.FloydEndElement = FloydEndElement;            
            var FloydCommand = (function (_super) {
                __extends(FloydCommand, _super);
                function FloydCommand(document, command) {
                                _super.call(this, document);
                    this.command = command;
                }
                FloydCommand.prototype.init = function () {
                    _super.prototype.init.call(this);
                    var group = new CodeCompass.Drawings.Raphael2D.Primitives.Group(this.document.getCanvas(), Point2D.Origo, this.document.getSize(this.size));
                    var rectangle = new primitives.Rectange(this.document.getCanvas(), Point2D.Origo, new CodeCompass.Drawings.RelativeSize(1, 1));
                    rectangle.border.color = Colors.CornflowerBlue;
                    rectangle.fill = Colors.White;
                    rectangle.border.size = 3;
                    var text = new primitives.Text(this.document.getCanvas(), this.command, Point2D.create(group.size.width / 2.0, group.size.height / 2.0 - 7.5), 15, "Verdana", Colors.DarkBlue);
                    text.horizontalAlign = CodeCompass.Drawings.HorizontalAlign.center;
                    this._element = group.add(rectangle, text);
                    this.inputPoints.push(CodeCompass.Drawings.Point2D.create(this.element.size.width / 2.0, 0));
                    this.outputPoints.push(CodeCompass.Drawings.Point2D.create(this.element.size.width / 2.0, this.element.size.height));
                };
                return FloydCommand;
            })(FloydElement);
            Diagram.FloydCommand = FloydCommand;            
            var FloydBlock = (function (_super) {
                __extends(FloydBlock, _super);
                function FloydBlock(document, children) {
                    if (typeof children === "undefined") { children = []; }
                                _super.call(this, document);
                    this._children = children;
                }
                Object.defineProperty(FloydBlock.prototype, "inputPoints", {
                    get: function () {
                        var firstElement = Enumerable.From(this._children).First();
                        if(Ensure.isDefined(firstElement)) {
                            return firstElement.inputPoints;
                        }
                        return [];
                    },
                    enumerable: true,
                    configurable: true
                });
                Object.defineProperty(FloydBlock.prototype, "outputPoints", {
                    get: function () {
                        var lastElement = Enumerable.From(this._children).Last();
                        if(Ensure.isDefined(lastElement)) {
                            return lastElement.outputPoints;
                        }
                        return [];
                    },
                    enumerable: true,
                    configurable: true
                });
                Object.defineProperty(FloydBlock.prototype, "children", {
                    get: function () {
                        return this._children;
                    },
                    enumerable: true,
                    configurable: true
                });
                FloydBlock.prototype.dispose = function () {
                    if(Ensure.isDefined(this._children)) {
                        this._children.forEach(function (e) {
                            return e.dispose();
                        });
                    }
                };
                FloydBlock.prototype.init = function () {
                    _super.prototype.init.call(this);
                    if(Ensure.isDefined(this._children)) {
                        this._children.forEach(function (e) {
                            return e.init();
                        });
                    }
                    this.inputPoints.push(CodeCompass.Drawings.Point2D.create(0, 0));
                    this.outputPoints.push(CodeCompass.Drawings.Point2D.create(0, 0));
                };
                return FloydBlock;
            })(FloydElement);
            Diagram.FloydBlock = FloydBlock;            
            var FloydCondition = (function (_super) {
                __extends(FloydCondition, _super);
                function FloydCondition(document, condition) {
                                _super.call(this, document);
                    this.condition = condition;
                }
                Object.defineProperty(FloydCondition.prototype, "trueCase", {
                    get: function () {
                        return this._trueCase;
                    },
                    set: function (value) {
                        Ensure.defined(value);
                        if(Ensure.hasValue(this._trueCase)) {
                            this._trueCase.dispose();
                        }
                        this._trueCase = value;
                        this._trueCase.init();
                    },
                    enumerable: true,
                    configurable: true
                });
                Object.defineProperty(FloydCondition.prototype, "falseCase", {
                    get: function () {
                        return this._falseCase;
                    },
                    set: function (value) {
                        Ensure.defined(value);
                        if(Ensure.hasValue(this._falseCase)) {
                            this._falseCase.dispose();
                        }
                        this._falseCase = value;
                        this._falseCase.init();
                    },
                    enumerable: true,
                    configurable: true
                });
                FloydCondition.prototype.init = function () {
                    _super.prototype.init.call(this);
                    var group = new CodeCompass.Drawings.Raphael2D.Primitives.Group(this.document.getCanvas(), Point2D.Origo, this.document.getSize(this.size));
                    var rhombus = new primitives.Rhombus(this.document.getCanvas(), Point2D.Origo, new CodeCompass.Drawings.RelativeSize(1, 1));
                    rhombus.border.color = Colors.CornflowerBlue;
                    rhombus.fill = Colors.White;
                    rhombus.border.size = 3;
                    var text = new primitives.Text(this.document.getCanvas(), this.condition, Point2D.create(group.size.width / 2.0, group.size.height / 2.0 - 7.5), 15, "Verdana", Colors.DarkBlue);
                    text.horizontalAlign = CodeCompass.Drawings.HorizontalAlign.center;
                    this._element = group.add(rhombus, text);
                    this.inputPoints.push(CodeCompass.Drawings.Point2D.create(this.element.size.width / 2.0, 0));
                    this.outputPoints.push(CodeCompass.Drawings.Point2D.create(-1, this.element.size.height / 2.0));
                    this.outputPoints.push(CodeCompass.Drawings.Point2D.create(this.element.size.width, this.element.size.height / 2.0));
                };
                return FloydCondition;
            })(FloydElement);
            Diagram.FloydCondition = FloydCondition;            
            var FloydLoop = (function (_super) {
                __extends(FloydLoop, _super);
                function FloydLoop(document, condition) {
                                _super.call(this, document);
                    this.condition = condition;
                }
                Object.defineProperty(FloydLoop.prototype, "core", {
                    get: function () {
                        return this._core;
                    },
                    set: function (value) {
                        Ensure.defined(value);
                        if(Ensure.hasValue(this._core)) {
                            this._core.dispose();
                        }
                        this._core = value;
                        this._core.init();
                    },
                    enumerable: true,
                    configurable: true
                });
                FloydLoop.prototype.init = function () {
                    _super.prototype.init.call(this);
                    var group = new CodeCompass.Drawings.Raphael2D.Primitives.Group(this.document.getCanvas(), Point2D.Origo, this.document.getSize(this.size));
                    var ellipse = new primitives.Ellipse(this.document.getCanvas(), this.document.getPoint(Point2D.create(2.5, 1.5)), new CodeCompass.Drawings.RelativeSize(1, 1));
                    ellipse.border.color = Colors.CornflowerBlue;
                    ellipse.fill = Colors.White;
                    ellipse.border.size = 3;
                    var text = new primitives.Text(this.document.getCanvas(), this.condition, Point2D.create(group.size.width / 2.0, group.size.height / 2.0 - 7.5), 15, "Verdana", Colors.DarkBlue);
                    text.horizontalAlign = CodeCompass.Drawings.HorizontalAlign.center;
                    this._element = group.add(ellipse, text);
                    this.inputPoints.push(CodeCompass.Drawings.Point2D.create(this.element.size.width / 2.0, 0));
                    this.inputPoints.push(CodeCompass.Drawings.Point2D.create(0, this.element.size.height / 2.0));
                    this.outputPoints.push(CodeCompass.Drawings.Point2D.create(this.element.size.width, this.element.size.height / 2.0));
                    this.outputPoints.push(CodeCompass.Drawings.Point2D.create(this.element.size.width / 2.0, this.element.size.height));
                };
                return FloydLoop;
            })(FloydElement);
            Diagram.FloydLoop = FloydLoop;            
            var FloydElementFactory = (function () {
                function FloydElementFactory(document) {
                    this._document = document;
                    this._children = [];
                }
                Object.defineProperty(FloydElementFactory.prototype, "children", {
                    get: function () {
                        return this._children;
                    },
                    enumerable: true,
                    configurable: true
                });
                Object.defineProperty(FloydElementFactory.prototype, "document", {
                    get: function () {
                        return this._document;
                    },
                    enumerable: true,
                    configurable: true
                });
                FloydElementFactory.prototype.updateElement = function (element) {
                    Ensure.defined(element);
                    element.previous = this.lastElement(function (v) {
                        v.next = element;
                    });
                    return element;
                };
                FloydElementFactory.prototype.clear = function () {
                    this._children = [];
                };
                FloydElementFactory.prototype.toBlock = function () {
                    return new FloydBlock(this._document, this._children);
                };
                FloydElementFactory.prototype.lastElement = function (func) {
                    if (typeof func === "undefined") { func = null; }
                    var last = Enumerable.From(this._children).LastOrDefault(undefined);
                    if(Ensure.isDefined(last) && Ensure.isDefined(func)) {
                        func(last);
                    }
                    return last;
                };
                FloydElementFactory.prototype.start = function () {
                    this._children.push(new FloydStartElement(this._document));
                    return this;
                };
                FloydElementFactory.prototype.end = function () {
                    this._children.push(this.updateElement(new FloydEndElement(this._document)));
                    return this;
                };
                FloydElementFactory.prototype.command = function (text) {
                    var _command = new FloydCommand(this._document, text);
                    this._children.push(this.updateElement(_command));
                    return this;
                };
                FloydElementFactory.prototype.loop = function (condition, core) {
                    var factory = new FloydElementFactory(this._document);
                    var loop = new FloydLoop(this._document, condition);
                    loop.core = core(factory).toBlock();
                    this._children.push(this.updateElement(loop));
                    return this;
                };
                FloydElementFactory.prototype.condition = function (condition, trueCase, falseCase) {
                    var factory = new FloydElementFactory(this._document);
                    var ifThenElse = new FloydCondition(this._document, condition);
                    ifThenElse.trueCase = trueCase(factory).toBlock();
                    factory = new FloydElementFactory(this._document);
                    ifThenElse.falseCase = falseCase(factory).toBlock();
                    var ifThenElseEnd = new FloydConditionEndElement(this._document);
                    this._children.push(this.updateElement(ifThenElse));
                    this._children.push(this.updateElement(ifThenElseEnd));
                    return this;
                };
                FloydElementFactory.prototype.block = function (children) {
                    var factory = new FloydElementFactory(this._document);
                    var block = children(factory).toBlock();
                    this._children.push(this.updateElement(block));
                    return this;
                };
                FloydElementFactory.prototype.fromJSON = function (json) {
                    var _this = this;
                    Ensure.defined(json);
                    json.forEach(function (value, index) {
                        var assert = function (condition, errorMsg) {
                            if(condition) {
                                console.error(errorMsg);
                            }
                        };
                        assert(!Safe(value).has("id"), "Error: Invalid format of file! Missing id at " + value.type + " type element!");
                        if(!Safe(value).has("type")) {
                            console.error("Error: Invalid format of file! Missing type at element with id " + value.id.toString() + "!");
                            return _this;
                        } else if(value.type !== "for" && value.type !== "do" && value.type !== "if" && value.type !== "command") {
                            console.error("Error: invalid format of file! Unknown type at element with id " + value.id.toString() + "!");
                            return _this;
                        }
                        var errorMsg = "";
                        var addError = function (condition, msg) {
                            if(condition) {
                                errorMsg += msg + "\n";
                            }
                        };
                        switch(value.type) {
                            case "command":
                                if(!Safe(value).has("text")) {
                                    console.error("Error: Invalid format of file! Missing text at element with id " + value.id.toString() + "!");
                                } else if(getClass(value.text) !== String) {
                                    console.error("Error: Invalid format of file! The text attribute is not a string at element with id " + value.id.toString() + "!");
                                } else {
                                    return _this.command(value.text);
                                }
                                break;
                            case "do":
                            case "for":
                                addError(!Safe(value).has("attributes"), "Missing attributes!");
                                addError(!Safe(value).has("block"), "Missing block!");
                                addError(!Safe(value.attributes).has("cond"), "Missing condition of loop!");
                                addError(getClass(value.block) !== Array, "Invalid format of file!");
                                addError(getClass(value.attributes) !== Object, "Invalid format of file!");
                                addError(getClass(value.attributes.cond) !== String, "Invalid format of file!");
                                return _this.loop(value.attributes.cond, function (core) {
                                    return core.fromJSON(value.block);
                                });
                            case "if":
                                addError(!Safe(value).has("block"), "Missing block!");
                                addError(!Safe(value).has("elseBlock"), "Missing else block!");
                                addError(!Safe(value).has("attributes"), "Missing attributes!");
                                addError(!Safe(value.attributes).has("cond"), "Missing condition!");
                                addError(getClass(value.attributes) !== Object, "Wrong format of attributes!");
                                addError(getClass(value.block) !== Array, "Wrong format of block!");
                                addError(getClass(value.elseBlock) !== Array, "Wrong format of else block!");
                                addError(getClass(value.attributes.cond) !== String, "Wrong format of condition!");
                                if(errorMsg !== "") {
                                    console.error("Error: Invalid format of file: element id: " + value.id.toString() + "\n" + errorMsg);
                                }
                                return _this.condition(value.attributes.cond, function (trueCase) {
                                    return trueCase.fromJSON(value.block);
                                }, function (falseCase) {
                                    return falseCase.fromJSON(value.elseBlock);
                                });
                        }
                    });
                    return this;
                };
                return FloydElementFactory;
            })();
            Diagram.FloydElementFactory = FloydElementFactory;            
        })(Floyd.Diagram || (Floyd.Diagram = {}));
        var Diagram = Floyd.Diagram;
    })(CodeCompass.Floyd || (CodeCompass.Floyd = {}));
    var Floyd = CodeCompass.Floyd;
})(CodeCompass || (CodeCompass = {}));
//@ sourceMappingURL=FloydDiagram.js.map
