///<reference path="Common.ts" />

module Reflection {
    export interface IType {
        name: string;
        fullName: string;
    }

    export class Type implements IType {
        private _fullName: string;
        private _name: string;

        public get name() { return this._name; }
        public get fullName() { return this._fullName; }

        private getShortName(fullName: string): string {
            return <string>Enumerable.From(fullName.split(".")).Last();
        }

        public static create(fullName: string, name: string = null) {
            return new Type(fullName, name);
        }

        constructor (fullName: string, name: string = null) {
            Ensure.notBlank(fullName, "fullName");

            if (Ensure.isBlank(name))
                this._name = this.getShortName(fullName);
            else
                this._name = name;
            this._fullName = fullName;
        }
    }

    /*  
        Example:
            class Color implements Common.IObject { 
                //...
                public static get class() { return "CodeCompass.Drawings.Color"; }
                getType() { return new Common.Type(Color.class); }
                //..
            }
    */
    export interface IObject {
        getType(): IType;
    }
}