var Common;
(function (Common) {
    (function (Ensure) {
        function isStrict(obj, func, propertyName, message) {
            if(!func(obj)) {
                var msg = "";
                if(!isBlank(propertyName)) {
                    msg += "[" + propertyName + "]: ";
                }
                if(!isBlank(message)) {
                    msg += message;
                }
                throw msg;
            }
            return Ensure;
        }
        function defined(obj, propertyName, message) {
            if (typeof propertyName === "undefined") { propertyName = ""; }
            if (typeof message === "undefined") { message = "Property cannot be undefined or null!"; }
            return isStrict(obj, function (o) {
                return isDefined(o) && isNotNull(o);
            }, propertyName, message);
        }
        Ensure.defined = defined;
        function notBlank(text, propertyName, message) {
            return isStrict(text, function (obj) {
                return !isBlank(obj);
            }, propertyName, message);
        }
        Ensure.notBlank = notBlank;
        function hasValue(obj) {
            return Ensure.isDefined(obj) && Ensure.isNotNull(obj);
        }
        Ensure.hasValue = hasValue;
        function isNotNull(obj) {
            return !Ensure.isNull(obj);
        }
        Ensure.isNotNull = isNotNull;
        function isNull(obj) {
            return obj === null;
        }
        Ensure.isNull = isNull;
        function isDefined(obj) {
            return obj !== undefined;
        }
        Ensure.isDefined = isDefined;
        function isUndefined(obj) {
            return !Ensure.isDefined(obj);
        }
        Ensure.isUndefined = isUndefined;
        function isFunction(obj) {
            return typeof (obj) === "function";
        }
        Ensure.isFunction = isFunction;
        function isBlank(text) {
            return !hasValue(text) || text === "";
        }
        Ensure.isBlank = isBlank;
        function toIntervallum(num, min, max) {
            if (typeof min === "undefined") { min = 0; }
            if (typeof max === "undefined") { max = 1; }
            Ensure.defined(min).defined(max).defined(num);
            if(num < min) {
                num = min;
            } else {
                if(num > max) {
                    num = max;
                }
            }
            return num;
        }
        Ensure.toIntervallum = toIntervallum;
    })(Common.Ensure || (Common.Ensure = {}));
    var Ensure = Common.Ensure;
        var SafeElement = (function () {
        function SafeElement(object) {
            this._object = null;
            this._object = object;
        }
        SafeElement.prototype.getObject = function () {
            return this._object;
        };
        SafeElement.prototype.has = function () {
            var _this = this;
            var property = [];
            for (var _i = 0; _i < (arguments.length - 0); _i++) {
                property[_i] = arguments[_i + 0];
            }
            return Enumerable.From(property).All(function (value) {
                return Ensure.hasValue(_this._object[value]);
            });
        };
        SafeElement.prototype.apply = function (func) {
            return new SafeElement(func(this._object));
        };
        SafeElement.prototype.tryGet = function (property, func) {
            if(this.has(property)) {
                func(this._object[property], this._object);
            }
            return this;
        };
        return SafeElement;
    })();    
    function Safe(object) {
        Ensure.defined(object);
        return new SafeElement(object);
    }
    Common.Safe = Safe;
    var LetElement = (function () {
        function LetElement(object) {
            this._object = object;
        }
        LetElement.prototype.default = function (value) {
            var result = this._object;
            if(Ensure.isNull(result) || Ensure.isUndefined(result)) {
                result = value;
            }
            return new LetElement(result);
        };
        LetElement.prototype.be = function (func) {
            return new LetElement(func(this._object));
        };
        Object.defineProperty(LetElement.prototype, "value", {
            get: function () {
                return this._object;
            },
            enumerable: true,
            configurable: true
        });
        return LetElement;
    })();    
    var LazyLetElement = (function () {
        function LazyLetElement(object, func) {
            if (typeof func === "undefined") { func = function (x) {
                return x;
            }; }
            this._object = object;
            this._func = func;
        }
        LazyLetElement.prototype.default = function (value) {
            var _this = this;
            return new LazyLetElement(this._object, function (x) {
                var result = _this._func(x);
                if(Ensure.isNull(result) || Ensure.isUndefined(result)) {
                    result = value;
                }
                return result;
            });
        };
        LazyLetElement.prototype.be = function (func) {
            var _this = this;
            return new LazyLetElement(this._object, function (value) {
                return func(_this._func(value));
            });
        };
        Object.defineProperty(LazyLetElement.prototype, "value", {
            get: function () {
                return this._func(this._object);
            },
            enumerable: true,
            configurable: true
        });
        return LazyLetElement;
    })();    
    function Let(object, lazy) {
        if (typeof lazy === "undefined") { lazy = false; }
        if(lazy) {
            return new LazyLetElement(object);
        } else {
            return new LetElement(object);
        }
    }
    Common.Let = Let;
})(Common || (Common = {}));
var Let = Common.Let;
var Ensure = Common.Ensure;
var Safe = Common.Safe;
var __extends = this.__extends || function (d, b) {
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
///<reference path="Common.ts" />
var CodeCompass;
(function (CodeCompass) {
    (function (Drawings) {
        var Color = (function () {
            function Color(red, green, blue, alpha) {
                if (typeof alpha === "undefined") { alpha = 1; }
                this._R = Ensure.toIntervallum(red, 0, 255);
                this._G = Ensure.toIntervallum(green, 0, 255);
                this._B = Ensure.toIntervallum(blue, 0, 255);
                this._A = Ensure.toIntervallum(alpha, 0, 1);
            }
            Object.defineProperty(Color.prototype, "R", {
                get: function () {
                    return this._R;
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Color.prototype, "G", {
                get: function () {
                    return this._G;
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Color.prototype, "B", {
                get: function () {
                    return this._B;
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Color.prototype, "A", {
                get: function () {
                    return this._A;
                },
                enumerable: true,
                configurable: true
            });
            Color.prototype.add = function (color) {
                return new Color(this.R + color.R, this.G + color.G, this.B + color.B, this.A + color.A);
            };
            Color.prototype.toCSS = function () {
                return "rgba(" + [
                    this.R, 
                    this.G, 
                    this.B, 
                    this.A
                ].join(",") + ")";
            }//TODO: fix hex code generator
            ;
            Color.prototype.toHex = function () {
                var hexCodes = [
                    "0", 
                    "1", 
                    "2", 
                    "3", 
                    "4", 
                    "5", 
                    "6", 
                    "7", 
                    "8", 
                    "9", 
                    "a", 
                    "b", 
                    "c", 
                    "d", 
                    "e", 
                    "f"
                ];
                var result = "#";
                [
                    this.R, 
                    this.G, 
                    this.B, 
                    this.A * 255
                ].forEach(function (value, index, array) {
                    if(value == 0) {
                        result += "00";
                    }
                    while(value != 0) {
                        result += hexCodes[value % 16];
                        value = Math.floor(value / 16);
                    }
                });
                return result;
            };
            return Color;
        })();
        Drawings.Color = Color;        
        var Size = (function () {
            function Size(width, height) {
                this.width = width;
                this.height = height;
            }
            Object.defineProperty(Size, "Zero", {
                get: function () {
                    return new Size(0, 0);
                },
                enumerable: true,
                configurable: true
            });
            return Size;
        })();
        Drawings.Size = Size;        
        var Point2D = (function () {
            function Point2D(x, y) {
                if (typeof x === "undefined") { x = 0; }
                if (typeof y === "undefined") { y = 0; }
                Ensure.defined(x, "x: number (Point2D)");
                Ensure.defined(y, "y: number (Point2D)");
                this._x = x;
                this._y = y;
            }
            Object.defineProperty(Point2D.prototype, "x", {
                get: function () {
                    return this._x;
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Point2D.prototype, "y", {
                get: function () {
                    return this._y;
                },
                enumerable: true,
                configurable: true
            });
            Point2D.create = function create(x, y) {
                if (typeof x === "undefined") { x = 0; }
                if (typeof y === "undefined") { y = 0; }
                return new Point2D(x, y);
            }
            Object.defineProperty(Point2D, "Origo", {
                get: function () {
                    return new Point2D(0, 0);
                },
                enumerable: true,
                configurable: true
            });
            Point2D.prototype.add = function (point) {
                Ensure.defined(point, "point", "Point cannot be null or undefined!");
                return new Point2D(this.x + point.x, this.y + point.y);
            };
            Point2D.prototype.distance = function (point) {
                if (typeof point === "undefined") { point = Point2D.Origo; }
                Ensure.defined(point, "point: IPoint2D (Point2D)", "Point cannot be null or undefined!");
                var sqr = function (x) {
                    return x * x;
                };
                return Math.sqrt(sqr(this.x - point.x) + sqr(this.y - point.y));
            };
            Point2D.prototype.negate = function () {
                return new Point2D(-this.x, -this.y);
            };
            return Point2D;
        })();
        Drawings.Point2D = Point2D;        
        var RelativePoint2D = (function (_super) {
            __extends(RelativePoint2D, _super);
            function RelativePoint2D(x, y, //This point can be an absolute and relative point.
            isAbsolute, horizontalAlign, verticalAlign) {
                if (typeof x === "undefined") { x = 0; }
                if (typeof y === "undefined") { y = 0; }
                if (typeof isAbsolute === "undefined") { isAbsolute = false; }
                if (typeof horizontalAlign === "undefined") { horizontalAlign = HorizontalAlign.left; }
                if (typeof verticalAlign === "undefined") { verticalAlign = VerticalAlign.top; }
                        _super.call(this, x, y);
                this.isAbsolute = isAbsolute;
                this.horizontalAlign = horizontalAlign;
                this.verticalAlign = verticalAlign;
            }
            return RelativePoint2D;
        })(Point2D);
        Drawings.RelativePoint2D = RelativePoint2D;        
        var RelativeSize = (function (_super) {
            __extends(RelativeSize, _super);
            function RelativeSize(width, height, isAbsolute) {
                if (typeof width === "undefined") { width = 0; }
                if (typeof height === "undefined") { height = 0; }
                if (typeof isAbsolute === "undefined") { isAbsolute = false; }
                        _super.call(this, width, height);
                this.width = width;
                this.height = height;
                this.isAbsolute = isAbsolute;
            }
            Object.defineProperty(RelativeSize, "Zero", {
                get: function () {
                    return new RelativeSize(0, 0);
                },
                enumerable: true,
                configurable: true
            });
            return RelativeSize;
        })(Size);
        Drawings.RelativeSize = RelativeSize;        
        (function (HorizontalAlign) {
            HorizontalAlign._map = [];
            HorizontalAlign._map[0] = "left";
            HorizontalAlign.left = 0;
            HorizontalAlign._map[1] = "center";
            HorizontalAlign.center = 1;
            HorizontalAlign._map[2] = "right";
            HorizontalAlign.right = 2;
        })(Drawings.HorizontalAlign || (Drawings.HorizontalAlign = {}));
        var HorizontalAlign = Drawings.HorizontalAlign;
        (function (VerticalAlign) {
            VerticalAlign._map = [];
            VerticalAlign._map[0] = "top";
            VerticalAlign.top = 0;
            VerticalAlign._map[1] = "center";
            VerticalAlign.center = 1;
            VerticalAlign._map[2] = "bottom";
            VerticalAlign.bottom = 2;
        })(Drawings.VerticalAlign || (Drawings.VerticalAlign = {}));
        var VerticalAlign = Drawings.VerticalAlign;
        (function (ArrowEnd) {
            ArrowEnd._map = [];
            ArrowEnd._map[0] = "classic";
            ArrowEnd.classic = 0;
            ArrowEnd._map[1] = "block";
            ArrowEnd.block = 1;
            ArrowEnd._map[2] = "open";
            ArrowEnd.open = 2;
            ArrowEnd._map[3] = "oval";
            ArrowEnd.oval = 3;
            ArrowEnd._map[4] = "diamond";
            ArrowEnd.diamond = 4;
            ArrowEnd._map[5] = "none";
            ArrowEnd.none = 5;
        })(Drawings.ArrowEnd || (Drawings.ArrowEnd = {}));
        var ArrowEnd = Drawings.ArrowEnd;
        ; ;
        (function (LineCap) {
            LineCap._map = [];
            LineCap._map[0] = "butt";
            LineCap.butt = 0;
            LineCap._map[1] = "square";
            LineCap.square = 1;
            LineCap._map[2] = "round";
            LineCap.round = 2;
        })(Drawings.LineCap || (Drawings.LineCap = {}));
        var LineCap = Drawings.LineCap;
        (function (LineJoin) {
            LineJoin._map = [];
            LineJoin._map[0] = "bevel";
            LineJoin.bevel = 0;
            LineJoin._map[1] = "round";
            LineJoin.round = 1;
            LineJoin._map[2] = "miter";
            LineJoin.miter = 2;
        })(Drawings.LineJoin || (Drawings.LineJoin = {}));
        var LineJoin = Drawings.LineJoin;
    })(CodeCompass.Drawings || (CodeCompass.Drawings = {}));
    var Drawings = CodeCompass.Drawings;
})(CodeCompass || (CodeCompass = {}));
var CodeCompass;
(function (CodeCompass) {
    (function (Drawings) {
            })(CodeCompass.Drawings || (CodeCompass.Drawings = {}));
    var Drawings = CodeCompass.Drawings;
})(CodeCompass || (CodeCompass = {}));
var CodeCompass;
(function (CodeCompass) {
    (function (Drawings) {
        var Colors = (function () {
            function Colors() { }
            Object.defineProperty(Colors, "AliceBlue", {
                get: function () {
                    return new Drawings.Color(240, 248, 255, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "AntiqueWhite", {
                get: function () {
                    return new Drawings.Color(250, 235, 215, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Aqua", {
                get: function () {
                    return new Drawings.Color(0, 255, 255, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Aquamarine", {
                get: function () {
                    return new Drawings.Color(127, 255, 212, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Azure", {
                get: function () {
                    return new Drawings.Color(240, 255, 255, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Beige", {
                get: function () {
                    return new Drawings.Color(245, 245, 220, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Bisque", {
                get: function () {
                    return new Drawings.Color(255, 228, 196, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Black", {
                get: function () {
                    return new Drawings.Color(0, 0, 0, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "BlanchedAlmond", {
                get: function () {
                    return new Drawings.Color(255, 235, 205, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Blue", {
                get: function () {
                    return new Drawings.Color(0, 0, 255, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "BlueViolet", {
                get: function () {
                    return new Drawings.Color(138, 43, 226, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Brown", {
                get: function () {
                    return new Drawings.Color(165, 42, 42, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "BurlyWood", {
                get: function () {
                    return new Drawings.Color(222, 184, 135, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "CadetBlue", {
                get: function () {
                    return new Drawings.Color(95, 158, 160, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Chartreuse", {
                get: function () {
                    return new Drawings.Color(127, 255, 0, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Chocolate", {
                get: function () {
                    return new Drawings.Color(210, 105, 30, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Coral", {
                get: function () {
                    return new Drawings.Color(255, 127, 80, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "CornflowerBlue", {
                get: function () {
                    return new Drawings.Color(100, 149, 237, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Cornsilk", {
                get: function () {
                    return new Drawings.Color(255, 248, 220, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Crimson", {
                get: function () {
                    return new Drawings.Color(220, 20, 60, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Cyan", {
                get: function () {
                    return new Drawings.Color(0, 255, 255, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "DarkBlue", {
                get: function () {
                    return new Drawings.Color(0, 0, 139, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "DarkCyan", {
                get: function () {
                    return new Drawings.Color(0, 139, 139, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "DarkGoldenrod", {
                get: function () {
                    return new Drawings.Color(184, 134, 11, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "DarkGray", {
                get: function () {
                    return new Drawings.Color(169, 169, 169, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "DarkGreen", {
                get: function () {
                    return new Drawings.Color(0, 100, 0, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "DarkKhaki", {
                get: function () {
                    return new Drawings.Color(189, 183, 107, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "DarkMagenta", {
                get: function () {
                    return new Drawings.Color(139, 0, 139, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "DarkOliveGreen", {
                get: function () {
                    return new Drawings.Color(85, 107, 47, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "DarkOrange", {
                get: function () {
                    return new Drawings.Color(255, 140, 0, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "DarkOrchid", {
                get: function () {
                    return new Drawings.Color(153, 50, 204, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "DarkRed", {
                get: function () {
                    return new Drawings.Color(139, 0, 0, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "DarkSalmon", {
                get: function () {
                    return new Drawings.Color(233, 150, 122, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "DarkSeaGreen", {
                get: function () {
                    return new Drawings.Color(143, 188, 143, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "DarkSlateBlue", {
                get: function () {
                    return new Drawings.Color(72, 61, 139, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "DarkSlateGray", {
                get: function () {
                    return new Drawings.Color(47, 79, 79, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "DarkTurquoise", {
                get: function () {
                    return new Drawings.Color(0, 206, 209, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "DarkViolet", {
                get: function () {
                    return new Drawings.Color(148, 0, 211, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "DeepPink", {
                get: function () {
                    return new Drawings.Color(255, 20, 147, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "DeepSkyBlue", {
                get: function () {
                    return new Drawings.Color(0, 191, 255, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "DimGray", {
                get: function () {
                    return new Drawings.Color(105, 105, 105, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "DodgerBlue", {
                get: function () {
                    return new Drawings.Color(30, 144, 255, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Firebrick", {
                get: function () {
                    return new Drawings.Color(178, 34, 34, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "FloralWhite", {
                get: function () {
                    return new Drawings.Color(255, 250, 240, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "ForestGreen", {
                get: function () {
                    return new Drawings.Color(34, 139, 34, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Fuchsia", {
                get: function () {
                    return new Drawings.Color(255, 0, 255, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Gainsboro", {
                get: function () {
                    return new Drawings.Color(220, 220, 220, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "GhostWhite", {
                get: function () {
                    return new Drawings.Color(248, 248, 255, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Gold", {
                get: function () {
                    return new Drawings.Color(255, 215, 0, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Goldenrod", {
                get: function () {
                    return new Drawings.Color(218, 165, 32, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Gray", {
                get: function () {
                    return new Drawings.Color(128, 128, 128, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Green", {
                get: function () {
                    return new Drawings.Color(0, 128, 0, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "GreenYellow", {
                get: function () {
                    return new Drawings.Color(173, 255, 47, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Honeydew", {
                get: function () {
                    return new Drawings.Color(240, 255, 240, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "HotPink", {
                get: function () {
                    return new Drawings.Color(255, 105, 180, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "IndianRed", {
                get: function () {
                    return new Drawings.Color(205, 92, 92, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Indigo", {
                get: function () {
                    return new Drawings.Color(75, 0, 130, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Ivory", {
                get: function () {
                    return new Drawings.Color(255, 255, 240, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Khaki", {
                get: function () {
                    return new Drawings.Color(240, 230, 140, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Lavender", {
                get: function () {
                    return new Drawings.Color(230, 230, 250, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "LavenderBlush", {
                get: function () {
                    return new Drawings.Color(255, 240, 245, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "LawnGreen", {
                get: function () {
                    return new Drawings.Color(124, 252, 0, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "LemonChiffon", {
                get: function () {
                    return new Drawings.Color(255, 250, 205, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "LightBlue", {
                get: function () {
                    return new Drawings.Color(173, 216, 230, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "LightCoral", {
                get: function () {
                    return new Drawings.Color(240, 128, 128, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "LightCyan", {
                get: function () {
                    return new Drawings.Color(224, 255, 255, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "LightGoldenrodYellow", {
                get: function () {
                    return new Drawings.Color(250, 250, 210, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "LightGray", {
                get: function () {
                    return new Drawings.Color(211, 211, 211, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "LightGreen", {
                get: function () {
                    return new Drawings.Color(144, 238, 144, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "LightPink", {
                get: function () {
                    return new Drawings.Color(255, 182, 193, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "LightSalmon", {
                get: function () {
                    return new Drawings.Color(255, 160, 122, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "LightSeaGreen", {
                get: function () {
                    return new Drawings.Color(32, 178, 170, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "LightSkyBlue", {
                get: function () {
                    return new Drawings.Color(135, 206, 250, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "LightSlateGray", {
                get: function () {
                    return new Drawings.Color(119, 136, 153, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "LightSteelBlue", {
                get: function () {
                    return new Drawings.Color(176, 196, 222, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "LightYellow", {
                get: function () {
                    return new Drawings.Color(255, 255, 224, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Lime", {
                get: function () {
                    return new Drawings.Color(0, 255, 0, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "LimeGreen", {
                get: function () {
                    return new Drawings.Color(50, 205, 50, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Linen", {
                get: function () {
                    return new Drawings.Color(250, 240, 230, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Magenta", {
                get: function () {
                    return new Drawings.Color(255, 0, 255, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Maroon", {
                get: function () {
                    return new Drawings.Color(128, 0, 0, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "MediumAquamarine", {
                get: function () {
                    return new Drawings.Color(102, 205, 170, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "MediumBlue", {
                get: function () {
                    return new Drawings.Color(0, 0, 205, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "MediumOrchid", {
                get: function () {
                    return new Drawings.Color(186, 85, 211, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "MediumPurple", {
                get: function () {
                    return new Drawings.Color(147, 112, 219, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "MediumSeaGreen", {
                get: function () {
                    return new Drawings.Color(60, 179, 113, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "MediumSlateBlue", {
                get: function () {
                    return new Drawings.Color(123, 104, 238, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "MediumSpringGreen", {
                get: function () {
                    return new Drawings.Color(0, 250, 154, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "MediumTurquoise", {
                get: function () {
                    return new Drawings.Color(72, 209, 204, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "MediumVioletRed", {
                get: function () {
                    return new Drawings.Color(199, 21, 133, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "MidnightBlue", {
                get: function () {
                    return new Drawings.Color(25, 25, 112, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "MintCream", {
                get: function () {
                    return new Drawings.Color(245, 255, 250, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "MistyRose", {
                get: function () {
                    return new Drawings.Color(255, 228, 225, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Moccasin", {
                get: function () {
                    return new Drawings.Color(255, 228, 181, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "NavajoWhite", {
                get: function () {
                    return new Drawings.Color(255, 222, 173, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Navy", {
                get: function () {
                    return new Drawings.Color(0, 0, 128, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "OldLace", {
                get: function () {
                    return new Drawings.Color(253, 245, 230, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Olive", {
                get: function () {
                    return new Drawings.Color(128, 128, 0, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "OliveDrab", {
                get: function () {
                    return new Drawings.Color(107, 142, 35, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Orange", {
                get: function () {
                    return new Drawings.Color(255, 165, 0, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "OrangeRed", {
                get: function () {
                    return new Drawings.Color(255, 69, 0, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Orchid", {
                get: function () {
                    return new Drawings.Color(218, 112, 214, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "PaleGoldenrod", {
                get: function () {
                    return new Drawings.Color(238, 232, 170, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "PaleGreen", {
                get: function () {
                    return new Drawings.Color(152, 251, 152, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "PaleTurquoise", {
                get: function () {
                    return new Drawings.Color(175, 238, 238, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "PaleVioletRed", {
                get: function () {
                    return new Drawings.Color(219, 112, 147, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "PapayaWhip", {
                get: function () {
                    return new Drawings.Color(255, 239, 213, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "PeachPuff", {
                get: function () {
                    return new Drawings.Color(255, 218, 185, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Peru", {
                get: function () {
                    return new Drawings.Color(205, 133, 63, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Pink", {
                get: function () {
                    return new Drawings.Color(255, 192, 203, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Plum", {
                get: function () {
                    return new Drawings.Color(221, 160, 221, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "PowderBlue", {
                get: function () {
                    return new Drawings.Color(176, 224, 230, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Purple", {
                get: function () {
                    return new Drawings.Color(128, 0, 128, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Red", {
                get: function () {
                    return new Drawings.Color(255, 0, 0, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "RosyBrown", {
                get: function () {
                    return new Drawings.Color(188, 143, 143, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "RoyalBlue", {
                get: function () {
                    return new Drawings.Color(65, 105, 225, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "SaddleBrown", {
                get: function () {
                    return new Drawings.Color(139, 69, 19, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Salmon", {
                get: function () {
                    return new Drawings.Color(250, 128, 114, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "SandyBrown", {
                get: function () {
                    return new Drawings.Color(244, 164, 96, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "SeaGreen", {
                get: function () {
                    return new Drawings.Color(46, 139, 87, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "SeaShell", {
                get: function () {
                    return new Drawings.Color(255, 245, 238, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Sienna", {
                get: function () {
                    return new Drawings.Color(160, 82, 45, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Silver", {
                get: function () {
                    return new Drawings.Color(192, 192, 192, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "SkyBlue", {
                get: function () {
                    return new Drawings.Color(135, 206, 235, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "SlateBlue", {
                get: function () {
                    return new Drawings.Color(106, 90, 205, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "SlateGray", {
                get: function () {
                    return new Drawings.Color(112, 128, 144, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Snow", {
                get: function () {
                    return new Drawings.Color(255, 250, 250, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "SpringGreen", {
                get: function () {
                    return new Drawings.Color(0, 255, 127, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "SteelBlue", {
                get: function () {
                    return new Drawings.Color(70, 130, 180, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Tan", {
                get: function () {
                    return new Drawings.Color(210, 180, 140, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Teal", {
                get: function () {
                    return new Drawings.Color(0, 128, 128, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Thistle", {
                get: function () {
                    return new Drawings.Color(216, 191, 216, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Tomato", {
                get: function () {
                    return new Drawings.Color(255, 99, 71, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Transparent", {
                get: function () {
                    return new Drawings.Color(255, 255, 255, 0);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Turquoise", {
                get: function () {
                    return new Drawings.Color(64, 224, 208, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Violet", {
                get: function () {
                    return new Drawings.Color(238, 130, 238, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Wheat", {
                get: function () {
                    return new Drawings.Color(245, 222, 179, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "White", {
                get: function () {
                    return new Drawings.Color(255, 255, 255, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "WhiteSmoke", {
                get: function () {
                    return new Drawings.Color(245, 245, 245, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "Yellow", {
                get: function () {
                    return new Drawings.Color(255, 255, 0, 1);
                },
                enumerable: true,
                configurable: true
            });
            Object.defineProperty(Colors, "YellowGreen", {
                get: function () {
                    return new Drawings.Color(154, 205, 50, 1);
                },
                enumerable: true,
                configurable: true
            });
            return Colors;
        })();
        Drawings.Colors = Colors;        
    })(CodeCompass.Drawings || (CodeCompass.Drawings = {}));
    var Drawings = CodeCompass.Drawings;
})(CodeCompass || (CodeCompass = {}));
///<reference path="../Scripts/raphael.d.ts" />
///<reference path="../Scripts/jquery-1.8.3.d.ts" />
///<reference path="../Scripts/linq.d.ts" />
///<reference path="Drawings.ts" />
var CodeCompass;
(function (CodeCompass) {
    (function (Drawings) {
        (function (Raphael2D) {
            var Canvas = (function () {
                function Canvas(element, width, height) {
                    if (typeof width === "undefined") { width = 1000; }
                    if (typeof height === "undefined") { height = width * 9 / 16; }
                    this.width = width;
                    this.height = height;
                    Ensure.defined(element, "element: string (Canvas)").defined(width, "width: number (Canvas)").defined(height, "height: number (Canvas)");
                    this._id = element;
                    this._paper = Raphael(element, width, height);
                }
                Object.defineProperty(Canvas.prototype, "id", {
                    get: function () {
                        return this._id;
                    },
                    set: function (value) {
                        this._id = value;
                        $("#" + value).attr("id", value);
                    },
                    enumerable: true,
                    configurable: true
                });
                Object.defineProperty(Canvas.prototype, "className", {
                    get: function () {
                        return this._className;
                    },
                    set: function (value) {
                        this._className = value;
                        $("." + value).attr("class", value);
                    },
                    enumerable: true,
                    configurable: true
                });
                Object.defineProperty(Canvas.prototype, "paper", {
                    get: function () {
                        return this._paper;
                    },
                    enumerable: true,
                    configurable: true
                });
                return Canvas;
            })();
            Raphael2D.Canvas = Canvas;            
            var Stroke = (function () {
                function Stroke(element) {
                    this.color = Drawings.Colors.Black;
                    this.miterlimit = 0;
                    this.size = 1;
                    this.linecap = Drawings.LineCap.butt;
                    this.linejoin = Drawings.LineJoin.bevel;
                    this.element = element;
                }
                Object.defineProperty(Stroke.prototype, "element", {
                    get: function () {
                        return this._element;
                    },
                    set: function (value) {
                        Ensure.defined(value, "element: RaphaelElement (Stroke)");
                        this._element = value;
                        this.refresh();
                    },
                    enumerable: true,
                    configurable: true
                });
                Stroke.prototype.refresh = function () {
                    if(Ensure.hasValue(this.element)) {
                        this.element.attr({
                            "stroke": Ensure.hasValue(this.color) ? this.color.toCSS() : "black",
                            "stroke-width": this.size,
                            "stroke-linecap": this.linecap,
                            "stroke-linejoin": this.linejoin,
                            "stroke-miterlimit": this.miterlimit
                        });
                    }
                };
                Object.defineProperty(Stroke.prototype, "color", {
                    get: function () {
                        return this._color;
                    },
                    set: function (value) {
                        if(Ensure.hasValue(value)) {
                            this._color = value;
                            if(Ensure.hasValue(this.element)) {
                                this.element.attr({
                                    "stroke": this.color.toCSS()
                                });
                            }
                        }
                    },
                    enumerable: true,
                    configurable: true
                });
                Object.defineProperty(Stroke.prototype, "size", {
                    get: function () {
                        return this._size;
                    },
                    set: function (value) {
                        if(Ensure.hasValue(value)) {
                            this._size = value;
                            if(Ensure.hasValue(this.element)) {
                                this.element.attr({
                                    "stroke-width": this.size
                                });
                            }
                        }
                    },
                    enumerable: true,
                    configurable: true
                });
                Object.defineProperty(Stroke.prototype, "linecap", {
                    get: function () {
                        return this._linecap;
                    },
                    set: function (value) {
                        if(Ensure.hasValue(value)) {
                            this._linecap = value;
                            if(Ensure.hasValue(this.element)) {
                                this.element.attr({
                                    "stroke-linecap": this.linecap
                                });
                            }
                        }
                    },
                    enumerable: true,
                    configurable: true
                });
                Object.defineProperty(Stroke.prototype, "linejoin", {
                    get: function () {
                        return this._linejoin;
                    },
                    set: function (value) {
                        if(Ensure.hasValue(value)) {
                            this._linejoin = value;
                            if(Ensure.hasValue(this.element)) {
                                this.element.attr({
                                    "stroke-linejoin": this.linejoin
                                });
                            }
                        }
                    },
                    enumerable: true,
                    configurable: true
                });
                Object.defineProperty(Stroke.prototype, "miterlimit", {
                    get: function () {
                        return this._miterlimit;
                    },
                    set: function (value) {
                        if(Ensure.hasValue(value)) {
                            this._miterlimit = value;
                            if(Ensure.hasValue(this.element)) {
                                this.element.attr({
                                    "stroke-miterlimit": this.miterlimit
                                });
                            }
                        }
                    },
                    enumerable: true,
                    configurable: true
                });
                return Stroke;
            })();
            Raphael2D.Stroke = Stroke;            
        })(Drawings.Raphael2D || (Drawings.Raphael2D = {}));
        var Raphael2D = Drawings.Raphael2D;
    })(CodeCompass.Drawings || (CodeCompass.Drawings = {}));
    var Drawings = CodeCompass.Drawings;
})(CodeCompass || (CodeCompass = {}));
var CodeCompass;
(function (CodeCompass) {
    (function (Drawings) {
        (function (Raphael2D) {
            (function (Utilities) {
                (function (Point2D) {
                    function getPath() {
                        var points = [];
                        for (var _i = 0; _i < (arguments.length - 0); _i++) {
                            points[_i] = arguments[_i + 0];
                        }
                        return getPathFromArray(points);
                    }
                    Point2D.getPath = getPath;
                    function getPathFromArray(points) {
                        var result = "M";
                        return Let(result).be(function (value) {
                            var output = [];
                            for(var index in points) {
                                output.push(points[index].x + " " + points[index].y);
                            }
                            value += output.join("L");
                            return value;
                        }).value;
                    }
                    Point2D.getPathFromArray = getPathFromArray;
                    function arrange(container, points) {
                        Ensure.defined(container).defined(points);
                        var list = Enumerable.From(points).Select(function (point, index) {
                            var absolute = true;
                            Safe(point).tryGet("isAbsolute", function (value) {
                                absolute = value;
                            });
                            var result = {
                                x: absolute ? point.x : point.x * container.width,
                                y: absolute ? point.y : point.y * container.height
                            };
                            Safe(point).tryGet("horizontalAlign", function (value) {
                                switch(value) {
                                    case Drawings.HorizontalAlign.left: {
                                        result.x = point.x * container.width;
                                        break;

                                    }
                                    case Drawings.HorizontalAlign.center: {
                                        result.x = absolute ? point.x + container.width / 2 : (0.5 - point.x) * container.width;
                                        break;

                                    }
                                    case Drawings.HorizontalAlign.right: {
                                        result.x = absolute ? point.x + container.width : (1 - point.x) * container.width;
                                        break;

                                    }
                                }
                            }).tryGet("verticalAlign", function (value) {
                                switch(value) {
                                    case Drawings.VerticalAlign.top: {
                                        result.y = point.y * container.height;
                                        break;

                                    }
                                    case Drawings.VerticalAlign.center: {
                                        result.y = absolute ? point.y + container.height / 2 : (0.5 - point.y) * container.height;
                                        break;

                                    }
                                    case Drawings.VerticalAlign.bottom: {
                                        result.y = absolute ? point.y + container.height : (1 - point.y) * container.height;
                                        break;

                                    }
                                }
                            });
                            return new CodeCompass.Drawings.Point2D(result.x, result.y);
                        }).ToArray();
                        return list;
                    }
                    Point2D.arrange = arrange;
                })(Utilities.Point2D || (Utilities.Point2D = {}));
                var Point2D = Utilities.Point2D;
                (function (Size) {
                    function arrange(container, sizes) {
                        var list = Enumerable.From(sizes).Select(function (size, index) {
                            var result;
                            if(Safe(size).has("isAbsolute")) {
                                result = new CodeCompass.Drawings.Size(container.width * size.width, container.height * size.height);
                            } else {
                                result = new CodeCompass.Drawings.Size(size.width, size.height);
                            }
                            return result;
                        }).ToArray();
                        return list;
                    }
                    Size.arrange = arrange;
                })(Utilities.Size || (Utilities.Size = {}));
                var Size = Utilities.Size;
            })(Raphael2D.Utilities || (Raphael2D.Utilities = {}));
            var Utilities = Raphael2D.Utilities;
        })(Drawings.Raphael2D || (Drawings.Raphael2D = {}));
        var Raphael2D = Drawings.Raphael2D;
    })(CodeCompass.Drawings || (CodeCompass.Drawings = {}));
    var Drawings = CodeCompass.Drawings;
})(CodeCompass || (CodeCompass = {}));
var CodeCompass;
(function (CodeCompass) {
    (function (Drawings) {
        (function (Raphael2D) {
            (function (Primitives) {
                var GraphicalObject = (function () {
                    function GraphicalObject(canvas, location, size) {
                        this.canvas = canvas;
                        this._isDisposed = false;
                        this._location = location;
                        this._visible = true;
                        this._size = size;
                    }
                    Object.defineProperty(GraphicalObject.prototype, "isDisposed", {
                        get: function () {
                            return this._isDisposed;
                        },
                        enumerable: true,
                        configurable: true
                    });
                    Object.defineProperty(GraphicalObject.prototype, "size", {
                        get: function () {
                            return this._size;
                        },
                        set: function (value) {
                            Ensure.defined(value);
                            this._size = value;
                            if(Ensure.isDefined(this._element)) {
                                this._element.attr({
                                    "width": value.width,
                                    "height": value.height,
                                    "rx": value.width / 2,
                                    "ry": value.height / 2
                                });
                            }
                        },
                        enumerable: true,
                        configurable: true
                    });
                    Object.defineProperty(GraphicalObject.prototype, "visible", {
                        get: function () {
                            return this._visible;
                        },
                        set: function (value) {
                            Ensure.defined(value, "value: bool (GraphicalObject)");
                            this._visible = value;
                            if(value) {
                                this.element.show();
                            } else {
                                this.element.hide();
                            }
                        },
                        enumerable: true,
                        configurable: true
                    });
                    Object.defineProperty(GraphicalObject.prototype, "element", {
                        get: function () {
                            return this._element;
                        },
                        set: function (value) {
                            this._element = value;
                        },
                        enumerable: true,
                        configurable: true
                    });
                    Object.defineProperty(GraphicalObject.prototype, "location", {
                        get: function () {
                            return this._location;
                        },
                        set: function (value) {
                            Ensure.defined(value, "value: IPoint2D (GraphicalObject)");
                            this._location = value;
                            this._element.attr({
                                "x": value.x,
                                "y": value.y,
                                "cx": value.x,
                                "cy": value.y
                            });
                        },
                        enumerable: true,
                        configurable: true
                    });
                    GraphicalObject.prototype.dispose = function () {
                        if(!this.isDisposed && Ensure.isDefined(this.element)) {
                            this.element.remove();
                            this._isDisposed = true;
                        }
                    };
                    return GraphicalObject;
                })();
                Primitives.GraphicalObject = GraphicalObject;                
                var Shape = (function (_super) {
                    __extends(Shape, _super);
                    function Shape(canvas, location, size) {
                        if (typeof location === "undefined") { location = Drawings.Point2D.Origo; }
                        if (typeof size === "undefined") { size = Drawings.Size.Zero; }
                                        _super.call(this, canvas, location, size);
                        Ensure.defined(canvas, "canvas: Canvas (Shape)").defined(location, "location: IPoint (Shape)").defined(size, "size: ISize (Shape)");
                        this._border = null;
                        this.fill = Drawings.Colors.Blue;
                    }
                    Object.defineProperty(Shape.prototype, "fill", {
                        get: function () {
                            return this._fill;
                        },
                        set: function (value) {
                            if(Ensure.hasValue(value)) {
                                this._fill = value;
                                if(Ensure.hasValue(this.element)) {
                                    this.element.attr({
                                        fill: value.toCSS()
                                    });
                                }
                            }
                        },
                        enumerable: true,
                        configurable: true
                    });
                    Object.defineProperty(Shape.prototype, "border", {
                        get: function () {
                            return this._border;
                        },
                        enumerable: true,
                        configurable: true
                    });
                    Object.defineProperty(Shape.prototype, "element", {
                        get: function () {
                            return this._element;
                        },
                        set: function (value) {
                            Ensure.defined(value, "value: RaphaelElement (Shape)");
                            this._element = value;
                            if(!Ensure.hasValue(this.border)) {
                                this._border = new Raphael2D.Stroke(value);
                            } else {
                                this.border.element = value;
                            }
                            this.element.attr({
                                fill: this.fill.toCSS()
                            });
                        },
                        enumerable: true,
                        configurable: true
                    });
                    return Shape;
                })(GraphicalObject);
                Primitives.Shape = Shape;                
                var Rectange = (function (_super) {
                    __extends(Rectange, _super);
                    function Rectange(canvas, location, size) {
                        if (typeof location === "undefined") { location = Drawings.Point2D.Origo; }
                        if (typeof size === "undefined") { size = Drawings.Size.Zero; }
                                        _super.call(this, canvas, location, size);
                        this.init();
                    }
                    Rectange.prototype.init = function () {
                        this.element = (this.canvas).paper.rect(this.location.x, this.location.y, this.size.width, this.size.height);
                    };
                    return Rectange;
                })(Shape);
                Primitives.Rectange = Rectange;                
                var Ellipse = (function (_super) {
                    __extends(Ellipse, _super);
                    function Ellipse(canvas, location, size) {
                        if (typeof location === "undefined") { location = Drawings.Point2D.Origo; }
                        if (typeof size === "undefined") { size = Drawings.Size.Zero; }
                                        _super.call(this, canvas, location, size);
                        this.init();
                    }
                    Ellipse.prototype.init = function () {
                        this.element = (this.canvas).paper.ellipse(this.location.x, this.location.y, this.size.width / 2, this.size.height / 2);
                    };
                    return Ellipse;
                })(Shape);
                Primitives.Ellipse = Ellipse;                
                var Polygon = (function (_super) {
                    __extends(Polygon, _super);
                    function Polygon(canvas, points, closed, location, size) {
                        if (typeof closed === "undefined") { closed = true; }
                        if (typeof location === "undefined") { location = Drawings.Point2D.Origo; }
                        if (typeof size === "undefined") { size = Drawings.Size.Zero; }
                                        _super.call(this, canvas, location, size);
                        this.points = points;
                        this._closed = false;
                        this._closed = closed;
                        this.init();
                    }
                    Object.defineProperty(Polygon.prototype, "isClosed", {
                        get: function () {
                            return this._closed;
                        },
                        enumerable: true,
                        configurable: true
                    });
                    Object.defineProperty(Polygon.prototype, "size", {
                        get: function () {
                            return this._size;
                        },
                        set: function (value) {
                            Ensure.defined(value);
                            this._size = value;
                            if(Ensure.isDefined(this.element)) {
                                this.init();
                            }
                        },
                        enumerable: true,
                        configurable: true
                    });
                    Object.defineProperty(Polygon.prototype, "location", {
                        get: function () {
                            return this._location;
                        },
                        set: function (value) {
                            Ensure.defined(value);
                            this._location = value;
                            this.element.translate(value.x, value.y);
                        },
                        enumerable: true,
                        configurable: true
                    });
                    Polygon.prototype.init = function () {
                        if(this.points.length === 0) {
                            return;
                        }
                        /*var points = Enumerable.From(this.points)
                        .Select((point: IPoint2D, index) => {
                        
                        var absolute = false;
                        Safe(point).tryGet("isAbsolute", value => {
                        absolute = value;
                        });
                        
                        var result: IPoint2D = new RelativePoint2D(point.x, point.y, absolute);
                        
                        Safe(point).tryGet("horizontalAlign", value => {
                        var x: number;
                        
                        switch (value) {
                        case HorizontalAlign.left:
                        x = point.x;
                        break;
                        case HorizontalAlign.center:
                        x = absolute ? this.size.width / 2.0 : 0.5 - point.x;
                        break;
                        case HorizontalAlign.right:
                        x = absolute ? this.size.width : 1.0 - point.x;
                        break;
                        }
                        result = new RelativePoint2D(x, result.y, absolute);
                        }).tryGet("verticalAlign", value => {
                        var y: number;
                        switch (value) {
                        case VerticalAlign.top:
                        y = point.y;
                        break;
                        case VerticalAlign.center:
                        y = absolute ? this.size.height / 2.0 : 0.5 - point.y;
                        break;
                        case VerticalAlign.bottom:
                        y = absolute ? this.size.height : 1.0 - point.y;
                        break;
                        }
                        result = new RelativePoint2D(result.x, y, absolute);
                        });
                        
                        if (absolute)
                        return result;
                        
                        return new Point2D(result.x * this.size.width, result.y * this.size.height);
                        })
                        .Select((point: IPoint2D, index: number) => {
                        var result = "L " + point.x + "," + point.y;
                        if (index === 0)
                        result = "M " + [point.x, point.y].join(",") + result;
                        return result;
                        })
                        .ToArray();*/
                        var points = Enumerable.From(Raphael2D.Utilities.Point2D.arrange(this.size, this.points)).Select(function (point, index) {
                            var result = "L " + point.x + "," + point.y;
                            if(index === 0) {
                                result = "M " + [
                                    point.x, 
                                    point.y
                                ].join(",") + result;
                            }
                            return result;
                        }).ToArray();
                        var path = points.join(" ");
                        if(this.isClosed) {
                            path += "Z";
                        }
                        this.element = (this.canvas).paper.path(path).translate(this.location.x, this.location.y);
                    };
                    return Polygon;
                })(Shape);
                Primitives.Polygon = Polygon;                
                var Rhombus = (function (_super) {
                    __extends(Rhombus, _super);
                    function Rhombus(canvas, location, size) {
                        if (typeof location === "undefined") { location = Drawings.Point2D.Origo; }
                        if (typeof size === "undefined") { size = new Drawings.Size(100, 100); }
                                        _super.call(this, canvas, [
                    {
                        x: 0,
                        y: 0,
                        verticalAlign: Drawings.VerticalAlign.top,
                        horizontalAlign: Drawings.HorizontalAlign.center,
                        isAbsolute: true
                    }, 
                    {
                        x: 0,
                        y: 0,
                        verticalAlign: Drawings.VerticalAlign.center,
                        horizontalAlign: Drawings.HorizontalAlign.right,
                        isAbsolute: true
                    }, 
                    {
                        x: 0,
                        y: 0,
                        verticalAlign: Drawings.VerticalAlign.bottom,
                        horizontalAlign: Drawings.HorizontalAlign.center,
                        isAbsolute: true
                    }, 
                    {
                        x: 0,
                        y: 0,
                        verticalAlign: Drawings.VerticalAlign.center,
                        horizontalAlign: Drawings.HorizontalAlign.left,
                        isAbsolute: true
                    }
                ], true, location, size);
                    }
                    return Rhombus;
                })(Polygon);
                Primitives.Rhombus = Rhombus;                
                var Arrow = (function (_super) {
                    __extends(Arrow, _super);
                    function Arrow(canvas, location, size) {
                        if (typeof location === "undefined") { location = Drawings.Point2D.Origo; }
                        if (typeof size === "undefined") { size = new Drawings.Size(100, 100); }
                                        _super.call(this, canvas, [
                    {
                        x: 0,
                        y: 0.3,
                        isAbsolute: false
                    }, 
                    {
                        x: 0.6,
                        y: 0.3,
                        isAbsolute: false
                    }, 
                    {
                        x: 0.6,
                        y: 0,
                        isAbsolute: false
                    }, 
                    {
                        x: 1,
                        y: 0.5,
                        isAbsolute: false
                    }, 
                    {
                        x: 0.6,
                        y: 1,
                        isAbsolute: false
                    }, 
                    {
                        x: 0.6,
                        y: 0.7,
                        isAbsolute: false
                    }, 
                    {
                        x: 0,
                        y: 0.7,
                        isAbsolute: false
                    }
                ], true, location, size);
                    }
                    return Arrow;
                })(Polygon);
                Primitives.Arrow = Arrow;                
                var Text = (function (_super) {
                    __extends(Text, _super);
                    function Text(canvas, text, location, fontSize, fontFamily, foreground, background) {
                        if (typeof location === "undefined") { location = Drawings.Point2D.Origo; }
                        if (typeof fontSize === "undefined") { fontSize = 10; }
                        if (typeof fontFamily === "undefined") { fontFamily = "Arial"; }
                        if (typeof foreground === "undefined") { foreground = Drawings.Colors.Black; }
                        if (typeof background === "undefined") { background = Drawings.Colors.Transparent; }
                                        _super.call(this, canvas, location, new Drawings.Size(0, 0));
                        this._text = text;
                        this._fontSize = fontSize;
                        this._fontFamily = fontFamily;
                        this._foreground = foreground;
                        this._background = background;
                        this.init();
                    }
                    Object.defineProperty(Text.prototype, "size", {
                        get: function () {
                            if(Ensure.isDefined(this.element)) {
                                var boundingBox = this.element.getBBox();
                                return new CodeCompass.Drawings.Size(boundingBox.width, boundingBox.height);
                            }
                            return CodeCompass.Drawings.Size.Zero;
                        },
                        enumerable: true,
                        configurable: true
                    });
                    Object.defineProperty(Text.prototype, "text", {
                        get: function () {
                            return this._text;
                        },
                        set: function (value) {
                            Ensure.defined(this.element).defined(value);
                            this._text = value;
                            this.element.attr("text", value);
                        },
                        enumerable: true,
                        configurable: true
                    });
                    Object.defineProperty(Text.prototype, "fontSize", {
                        get: function () {
                            return this._fontSize;
                        },
                        set: function (value) {
                            Ensure.defined(value).defined(this.element);
                            this._fontSize = value;
                            this.element.attr({
                                "font-size": value
                            });
                        },
                        enumerable: true,
                        configurable: true
                    });
                    Object.defineProperty(Text.prototype, "fontFamily", {
                        get: function () {
                            return this._fontFamily;
                        },
                        set: function (value) {
                            Ensure.notBlank(value).defined(this.element);
                            this._fontFamily = value;
                            this.element.attr({
                                "font-family": value
                            });
                        },
                        enumerable: true,
                        configurable: true
                    });
                    Object.defineProperty(Text.prototype, "foreground", {
                        get: function () {
                            return this._foreground;
                        },
                        set: function (value) {
                            Ensure.defined(this.element).defined(value);
                            this._foreground = value;
                            this.element.attr({
                                "fill": value.toCSS()
                            });
                        },
                        enumerable: true,
                        configurable: true
                    });
                    Object.defineProperty(Text.prototype, "horizontalAlign", {
                        get: function () {
                            return this._horizontalAlign;
                        },
                        set: function (value) {
                            Ensure.defined(value);
                            var position;
                            switch(value) {
                                case Drawings.HorizontalAlign.left: {
                                    position = "start";
                                    break;

                                }
                                case Drawings.HorizontalAlign.center: {
                                    position = "middle";
                                    break;

                                }
                                case Drawings.HorizontalAlign.right: {
                                    position = "end";
                                    break;

                                }
                            }
                            this.element.attr({
                                "text-anchor": position
                            });
                            this._horizontalAlign = value;
                        },
                        enumerable: true,
                        configurable: true
                    });
                    Object.defineProperty(Text.prototype, "location", {
                        get: function () {
                            return this._location;
                        },
                        set: function (value) {
                            Ensure.defined(value);
                            this._location = new Drawings.Point2D(value.x, value.y);
                            Ensure.defined(this.element);
                            this.element.attr("x", value.x);
                            this.element.attr({
                                "y": value.y + this.fontSize / 2
                            });
                        },
                        enumerable: true,
                        configurable: true
                    });
                    Text.prototype.init = function () {
                        this.element = (this.canvas).paper.text(this.location.x, this.location.y, this.text);
                        this.element.attr({
                            "font-family": this.fontFamily,
                            "font-size": this.fontSize,
                            "text-anchor": "start",
                            "fill": this.foreground.toCSS(),
                            "y": this.location.y + this.fontSize / 2
                        });
                        var boundingBox = this.element.getBBox();
                        this.size = new Drawings.Size(boundingBox.width, boundingBox.height);
                    };
                    return Text;
                })(GraphicalObject);
                Primitives.Text = Text;                
                var Group = (function (_super) {
                    __extends(Group, _super);
                    function Group(canvas, location, size) {
                        if (typeof location === "undefined") { location = Drawings.Point2D.Origo; }
                        if (typeof size === "undefined") { size = new CodeCompass.Drawings.Size(0, 0); }
                                        _super.call(this, canvas, location, size);
                        this.canvas = canvas;
                        this._isDisposed = false;
                        this._size = size;
                        this.init();
                    }
                    Object.defineProperty(Group.prototype, "isDisposed", {
                        get: function () {
                            return this._isDisposed;
                        },
                        enumerable: true,
                        configurable: true
                    });
                    Object.defineProperty(Group.prototype, "size", {
                        get: function () {
                            return this._size;
                        },
                        set: function (value) {
                            Ensure.defined(value);
                            this._size = value;
                            this.update();
                        },
                        enumerable: true,
                        configurable: true
                    });
                    Object.defineProperty(Group.prototype, "visible", {
                        get: function () {
                            return this._visible;
                        },
                        set: function (value) {
                            Ensure.defined(value);
                            this._visible = value;
                            for(var child in this.children) {
                                this.children[child].visible = value;
                            }
                        },
                        enumerable: true,
                        configurable: true
                    });
                    Object.defineProperty(Group.prototype, "location", {
                        get: function () {
                            return this._location;
                        },
                        set: function (value) {
                            Ensure.defined(value);
                            this._location = value;
                            for(var child in this.children) {
                                var item = this.children[child];
                                var location = item.location;
                                Ensure.defined(location, "location");
                                item.location = location.add(value);
                            }
                        },
                        enumerable: true,
                        configurable: true
                    });
                    Object.defineProperty(Group.prototype, "children", {
                        get: function () {
                            return this._children;
                        },
                        enumerable: true,
                        configurable: true
                    });
                    Group.prototype.dispose = function () {
                        if(!this.isDisposed && Ensure.isDefined(this._children)) {
                            this._children.forEach(function (child) {
                                return child.dispose();
                            });
                            this._isDisposed = true;
                        }
                    };
                    Group.prototype.update = function (list) {
                        if (typeof list === "undefined") { list = this._children; }
                        var _this = this;
                        if(Ensure.isDefined(list)) {
                            list.forEach(function (child) {
                                child.location = Enumerable.From(Raphael2D.Utilities.Point2D.arrange(_this.size, [
                                    child.location
                                ])).First();
                                child.size = Enumerable.From(Raphael2D.Utilities.Size.arrange(_this.size, [
                                    child.size
                                ])).First();
                            });
                        }
                    }///TODO: remove Raphael objects from canvas
                    ;
                    Group.prototype.clear = function () {
                        return this;
                    };
                    Group.prototype.ungroup = function () {
                        var _this = this;
                        Let(this.location).be(function (value) {
                            var location = value;
                            _this.location = Drawings.Point2D.Origo;
                            _this._children = [];
                            _this._set.clear();
                            _this.location = location;
                        });
                        return this;
                    };
                    Group.prototype.add = function () {
                        var _this = this;
                        var children = [];
                        for (var _i = 0; _i < (arguments.length - 0); _i++) {
                            children[_i] = arguments[_i + 0];
                        }
                        Ensure.defined(children, "child: GraphicalObject (Group)");
                        children.forEach(function (value) {
                            _this._children.push(value);
                            _this._set.push(value.element);
                        });
                        this.update(children);
                        return this;
                    };
                    Group.prototype.init = function () {
                        this._children = [];
                        this._set = (this.canvas).paper.set();
                    };
                    return Group;
                })(GraphicalObject);
                Primitives.Group = Group;                
                var Line2D = (function (_super) {
                    __extends(Line2D, _super);
                    function Line2D(canvas, start, end) {
                                        _super.call(this, canvas, start, new Drawings.Size(Math.abs(end.add(start.negate()).x), Math.abs(end.add(start.negate()).y)));
                        this._start = start;
                        this._end = end;
                        this.init();
                    }
                    Object.defineProperty(Line2D.prototype, "arrowEnd", {
                        get: function () {
                            return this._arrowEnd;
                        },
                        set: function (value) {
                            Ensure.defined(value).defined(this.element, "this.element: RaphaelElement (Line2D)");
                            this._arrowEnd = value;
                            switch(value) {
                                case Drawings.ArrowEnd.block: {
                                    this.element.attr("arrow-end", "block");
                                    break;

                                }
                                case Drawings.ArrowEnd.classic: {
                                    this.element.attr("arrow-end", "classic");
                                    break;

                                }
                                case Drawings.ArrowEnd.diamond: {
                                    this.element.attr("arrow-end", "diamond");
                                    break;

                                }
                                case Drawings.ArrowEnd.none: {
                                    this.element.attr("arrow-end", "none");
                                    break;

                                }
                                case Drawings.ArrowEnd.open: {
                                    this.element.attr("arrow-end", "open");
                                    break;

                                }
                                case Drawings.ArrowEnd.oval: {
                                    this.element.attr("arrow-end", "oval");
                                    break;

                                }
                            }
                        },
                        enumerable: true,
                        configurable: true
                    });
                    Object.defineProperty(Line2D.prototype, "start", {
                        get: function () {
                            return this._start;
                        },
                        set: function (value) {
                            Ensure.defined(value).defined(this.element);
                            this._start = value;
                            this.location = value;
                            this.element.attr("path", Raphael2D.Utilities.Point2D.getPath(this.start, this.end));
                        },
                        enumerable: true,
                        configurable: true
                    });
                    Object.defineProperty(Line2D.prototype, "end", {
                        get: function () {
                            return this._end;
                        },
                        set: function (value) {
                            Ensure.defined(value).defined(this.element);
                            this._end = value;
                            this.element.attr("path", Raphael2D.Utilities.Point2D.getPath(this.start, this.end));
                        },
                        enumerable: true,
                        configurable: true
                    });
                    Object.defineProperty(Line2D.prototype, "stroke", {
                        get: function () {
                            return this._stroke;
                        },
                        enumerable: true,
                        configurable: true
                    });
                    Line2D.prototype.init = function () {
                        this.element = (this.canvas).paper.path(Raphael2D.Utilities.Point2D.getPath(this.start, this.end));
                        this._stroke = new Raphael2D.Stroke(this.element);
                        this.arrowEnd = Drawings.ArrowEnd.none;
                        this.element.attr("stroke-width", 4);
                    };
                    return Line2D;
                })(GraphicalObject);
                Primitives.Line2D = Line2D;                
                //TODO: support relative Points too!!!
                var MultiLine2D = (function (_super) {
                    __extends(MultiLine2D, _super);
                    function MultiLine2D(canvas, start, end) {
                        if (typeof start === "undefined") { start = Drawings.Point2D.Origo; }
                        if (typeof end === "undefined") { end = Drawings.Point2D.Origo; }
                                        _super.call(this, canvas, start, end);
                        this.start = start;
                        this.end = end;
                    }
                    MultiLine2D.prototype.getPoints = function () {
                        return Enumerable.From(this._points);
                    };
                    MultiLine2D.prototype.setPoints = function (value) {
                        Ensure.defined(value).defined(this.element);
                        this._points = value;
                        this.element.attr("path", Raphael2D.Utilities.Point2D.getPathFromArray(value));
                    };
                    return MultiLine2D;
                })(Line2D);
                Primitives.MultiLine2D = MultiLine2D;                
            })(Raphael2D.Primitives || (Raphael2D.Primitives = {}));
            var Primitives = Raphael2D.Primitives;
        })(Drawings.Raphael2D || (Drawings.Raphael2D = {}));
        var Raphael2D = Drawings.Raphael2D;
    })(CodeCompass.Drawings || (CodeCompass.Drawings = {}));
    var Drawings = CodeCompass.Drawings;
})(CodeCompass || (CodeCompass = {}));
///<reference path="Common.ts" />
var Reflection;
(function (Reflection) {
    var Type = (function () {
        function Type(fullName, name) {
            if (typeof name === "undefined") { name = null; }
            Ensure.notBlank(fullName, "fullName");
            if(Ensure.isBlank(name)) {
                this._name = this.getShortName(fullName);
            } else {
                this._name = name;
            }
            this._fullName = fullName;
        }
        Object.defineProperty(Type.prototype, "name", {
            get: function () {
                return this._name;
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(Type.prototype, "fullName", {
            get: function () {
                return this._fullName;
            },
            enumerable: true,
            configurable: true
        });
        Type.prototype.getShortName = function (fullName) {
            return Enumerable.From(fullName.split(".")).Last();
        };
        Type.create = function create(fullName, name) {
            if (typeof name === "undefined") { name = null; }
            return new Type(fullName, name);
        }
        return Type;
    })();
    Reflection.Type = Type;    
})(Reflection || (Reflection = {}));
///<reference path="Common.ts" />
///<reference path="Drawings.ts" />
///<reference path="DrawingsRaphael.ts" />
///<reference path="../Scripts/linq.d.ts" />
var CodeCompass;
(function (CodeCompass) {
    (function (Floyd) {
        (function (Diagram) {
            var drawings = CodeCompass.Drawings;
            var Point2D = drawings.Point2D;
            var Size = drawings.Size;
            var primitives = CodeCompass.Drawings.Raphael2D.Primitives;
            var utilities = CodeCompass.Drawings.Raphael2D.Utilities;
            var Colors = drawings.Colors;
            var FloydGraph = (function () {
                function FloydGraph() { }
                FloydGraph.prototype.getNodes = function (location, size, func) {
                    Ensure.defined(location).defined(size).defined(func);
                    for(var i = Math.max(location.x, 0); i < Math.min(location.x + size.width, this._graphSize.width); ++i) {
                        for(var j = Math.max(location.y, 0); j < Math.min(location.y + size.height, this._graphSize.height); ++j) {
                            this.nodes[j][i] = func(this.nodes[j][i], new CodeCompass.Drawings.Point2D(i, j));
                        }
                    }
                    return this;
                };
                FloydGraph.prototype.initNodes = function (size) {
                    Ensure.defined(size).defined(size.width).defined(size.height);
                    this._graphSize = size;
                    this.nodes = [];
                    for(var i = 0; i < size.width; ++i) {
                        this.nodes.push([]);
                        for(var j = 0; j < size.height; ++j) {
                            this.nodes[this.nodes.length - 1].push(1);
                        }
                    }
                    return this;
                }//Bellman-Ford
                ;
                FloydGraph.prototype.planRoute = function (start, end) {
                    var _this = this;
                    Ensure.defined(start).defined(end);
                    var distance = [];
                    var parent = [];
                    var isChanged = true;
                    for(var i = 0; i < this._graphSize.width; ++i) {
                        distance.push([]);
                        parent.push([]);
                        for(var j = 0; j < this._graphSize.height; ++j) {
                            distance[i].push(Number.POSITIVE_INFINITY);
                            parent[i].push(null);
                        }
                    }
                    distance[start.y][start.x] = 0;
                    var checkEdge = function (u, v) {
                        if(v.x < 0 || v.x >= _this._graphSize.width || v.y < 0 || v.y >= _this._graphSize.height) {
                            return;
                        }
                        if(distance[v.y][v.x] > distance[u.y][u.x] + _this.nodes[u.y][u.x]) {
                            distance[v.y][v.x] = distance[u.y][u.x] + _this.nodes[u.y][u.x];
                            parent[v.y][v.x] = {
                                x: u.x,
                                y: u.y
                            };
                            isChanged = true;
                        }
                    };
                    for(var k = 0; k <= this._graphSize.width * this._graphSize.height - 1 && isChanged; ++k) {
                        isChanged = false;
                        for(var i = 0; i < this._graphSize.width; ++i) {
                            for(var j = 0; j < this._graphSize.height; ++j) {
                                [
                                    {
                                        x: i - 1,
                                        y: j
                                    }, 
                                    {
                                        x: i + 1,
                                        y: j
                                    }, 
                                    {
                                        x: i,
                                        y: j - 1
                                    }, 
                                    {
                                        x: i,
                                        y: j + 1
                                    }
                                ].forEach(function (value) {
                                    var u = {
                                        x: i,
                                        y: j
                                    };
                                    checkEdge(u, value);
                                });
                            }
                        }
                    }
                    var route = [
                        end
                    ];
                    var p = parent[end.y][end.x];
                    this.addObject(end, new Size(1, 1), 100);
                    while(p !== null) {
                        this.addObject(p, new Size(1, 1), 100);
                        route.push(p);
                        p = parent[p.y][p.x];
                    }
                    return route.reverse();
                };
                FloydGraph.prototype.removeObject = function (location, size) {
                    this.getNodes(location, size, function (node) {
                        return 1;
                    });
                    return this;
                };
                FloydGraph.prototype.addObject = function (location, size, weight) {
                    if (typeof weight === "undefined") { weight = 10000; }
                    this.getNodes(location, size, function (node) {
                        return weight;
                    });
                    return this;
                };
                return FloydGraph;
            })();
            Diagram.FloydGraph = FloydGraph;            
            var FloydDocument = (function () {
                function FloydDocument(elementName) {
                    if (typeof elementName === "undefined") { elementName = "canvas"; }
                    this._isDisposed = false;
                    Ensure.notBlank(elementName, "elementName: string (FloydDocument)");
                    this._canvasSize = new Size(1000, 1000);
                    this._canvas = new CodeCompass.Drawings.Raphael2D.Canvas(elementName, this._canvasSize.width, this._canvasSize.height);
                    this._size = new Size(40, 40);
                    this._graph = new FloydGraph();
                    this._graph.initNodes(this._size);
                    this._children = [];
                }
                Object.defineProperty(FloydDocument.prototype, "isDisposed", {
                    get: function () {
                        return this._isDisposed;
                    },
                    enumerable: true,
                    configurable: true
                });
                Object.defineProperty(FloydDocument.prototype, "children", {
                    get: function () {
                        return this._children;
                    },
                    enumerable: true,
                    configurable: true
                });
                Object.defineProperty(FloydDocument.prototype, "start", {
                    get: function () {
                        return this._start;
                    },
                    enumerable: true,
                    configurable: true
                });
                Object.defineProperty(FloydDocument.prototype, "end", {
                    get: function () {
                        return this._end;
                    },
                    enumerable: true,
                    configurable: true
                });
                Object.defineProperty(FloydDocument.prototype, "size", {
                    get: function () {
                        return this._size;
                    },
                    enumerable: true,
                    configurable: true
                });
                FloydDocument.prototype.getCanvas = function () {
                    return this._canvas;
                };
                FloydDocument.prototype.fromPoint = function (location) {
                    return Point2D.create(Math.floor(location.x / (this._canvasSize.width / this.size.width)), Math.floor(location.y / (this._canvasSize.height / this.size.height)));
                };
                FloydDocument.prototype.getPoint = function (location) {
                    return Point2D.create(location.x * (this._canvasSize.width / this.size.width), location.y * (this._canvasSize.height / this.size.height));
                };
                FloydDocument.prototype.getSize = function (size) {
                    return new Size(size.width * (this._canvasSize.width / this.size.width), size.height * (this._canvasSize.height / this.size.height));
                };
                FloydDocument.prototype.clear = function () {
                    this._children = [];
                    return this;
                };
                FloydDocument.prototype.connect = function (start, end) {
                    var _this = this;
                    Ensure.defined(start).defined(end);
                    var startPoint = start.outputPoints[0].add(start.element.location);
                    var endPoint = end.inputPoints[0].add(end.element.location);
                    var route = this._graph.planRoute(this.fromPoint(startPoint), this.fromPoint(endPoint));
                    var arrow = new CodeCompass.Drawings.Raphael2D.Primitives.MultiLine2D(this._canvas, startPoint, endPoint);
                    arrow.arrowEnd = CodeCompass.Drawings.ArrowEnd.block;
                    arrow.stroke.size = 3;
                    this._arrows.push(arrow);
                    var points = Enumerable.From(route).TakeExceptLast().Select(//.Skip(1)
                    function (value) {
                        return _this.getPoint(new CodeCompass.Drawings.Point2D(value.x, value.y));
                    }).Select(function (value) {
                        return CodeCompass.Drawings.Point2D.create(value.x + _this._canvasSize.width / _this.size.width / 2, value.y + _this._canvasSize.height / _this.size.height / 2);
                    }).ToArray();
                    points.unshift(startPoint);
                    points.push(endPoint);
                    arrow.setPoints(points);
                };
                FloydDocument.prototype.arrangeLayout = function () {
                    var _this = this;
                    /*this._graph = new FloydGraph();
                    this._graph.initNodes(this._size);
                    
                    this._children.forEach((element: FloydElement, counter: number) => {
                    element.location = new Point2D(counter * 5 + 3, counter * 2 + 1);
                    this._graph.addObject(element.location, element.size);
                    });
                    
                    var start = <Drawings.Point2D>this._children[0].inputPoints[0].add(this._children[0].element.location);
                    var end = <Drawings.Point2D>this._children[0].inputPoints[1].add(this._children[0].element.location);
                    
                    var route = this._graph.planRoute(this.fromPoint(start), this.fromPoint(end));
                    
                    var arrow = new Drawings.Raphael2D.Primitives.MultiLine2D(this._canvas, Drawings.Point2D.Origo,
                    new Drawings.Point2D(35, 3));
                    
                    arrow.arrowEnd = Drawings.ArrowEnd.block;
                    arrow.stroke.size = 3;
                    
                    var points = Enumerable.From(route)
                    .Skip(1)
                    //.TakeExceptLast()
                    .Select((value: Drawings.IPoint2D) => this.getPoint(new Drawings.Point2D(value.x, value.y)))
                    .Select((value: Drawings.IPoint2D) => Drawings.Point2D.create(
                    value.x + this._canvasSize.width / this.size.width / 2.0,
                    value.y + this._canvasSize.height / this.size.height / 2.0))
                    .ToArray();
                    
                    points.unshift(start);
                    points.push(end);
                    
                    arrow.setPoints(points);*/
                    this._graph = new FloydGraph();
                    this._graph.initNodes(this._size);
                    this._arrows = [];
                    this._children.forEach(function (element, counter) {
                        //element.location = new Point2D(counter * 2, counter * 6 + 1);
                        element.location = new Point2D(Math.round(Math.random() * 20), Math.round(Math.random() * 20));
                        _this._graph.addObject(element.location, element.size);
                    });
                    this._children.forEach(function (child) {
                        if(Ensure.hasValue(child) && Ensure.hasValue(child.next)) {
                            _this.connect(child, child.next);
                        }
                    });
                    return this;
                };
                FloydDocument.prototype.init = function () {
                    Ensure.defined(this._children);
                    this._children.forEach(function (e) {
                        return e.init();
                    });
                };
                FloydDocument.prototype.dispose = function () {
                    if(!this._isDisposed && Ensure.isDefined(this._children)) {
                        this._children.forEach(function (e) {
                            return e.dispose();
                        });
                        this._isDisposed = true;
                    }
                };
                return FloydDocument;
            })();
            Diagram.FloydDocument = FloydDocument;            
            var FloydElement = (function () {
                function FloydElement(document, previous, next) {
                    if (typeof previous === "undefined") { previous = null; }
                    if (typeof next === "undefined") { next = null; }
                    this.previous = previous;
                    this.next = next;
                    Ensure.defined(document, "document: FloydDocument (FloydElement)");
                    this._document = document;
                    this._inputPoints = [];
                    this._outputPoints = [];
                    this._size = new CodeCompass.Drawings.Size(5, 3);
                    this._isDisposed = true;
                }
                Object.defineProperty(FloydElement.prototype, "element", {
                    get: function () {
                        return this._element;
                    },
                    enumerable: true,
                    configurable: true
                });
                Object.defineProperty(FloydElement.prototype, "inputPoints", {
                    get: function () {
                        return this._inputPoints;
                    },
                    enumerable: true,
                    configurable: true
                });
                Object.defineProperty(FloydElement.prototype, "outputPoints", {
                    get: function () {
                        return this._outputPoints;
                    },
                    enumerable: true,
                    configurable: true
                });
                Object.defineProperty(FloydElement.prototype, "size", {
                    get: function () {
                        return this._size;
                    },
                    enumerable: true,
                    configurable: true
                });
                Object.defineProperty(FloydElement.prototype, "location", {
                    get: function () {
                        return this._location;
                    },
                    set: function (value) {
                        Ensure.defined(value);
                        this._location = value;
                        this.element.location = this._document.getPoint(value);
                    },
                    enumerable: true,
                    configurable: true
                });
                FloydElement.prototype.init = function () {
                    this.dispose();
                    this._isDisposed = false;
                };
                FloydElement.prototype.dispose = function () {
                    if(Ensure.isDefined(this._element)) {
                        this._element.dispose();
                        this._isDisposed = true;
                    }
                };
                Object.defineProperty(FloydElement.prototype, "document", {
                    get: function () {
                        return this._document;
                    },
                    enumerable: true,
                    configurable: true
                });
                Object.defineProperty(FloydElement.prototype, "isDisposed", {
                    get: function () {
                        return this._isDisposed;
                    },
                    enumerable: true,
                    configurable: true
                });
                return FloydElement;
            })();
            Diagram.FloydElement = FloydElement;            
            var FloydStartElement = (function (_super) {
                __extends(FloydStartElement, _super);
                function FloydStartElement(document) {
                                _super.call(this, document);
                }
                FloydStartElement.prototype.init = function () {
                    _super.prototype.init.call(this);
                    var group = new CodeCompass.Drawings.Raphael2D.Primitives.Group(this.document.getCanvas(), Point2D.Origo, this.document.getSize(this.size));
                    var ellipse = new primitives.Ellipse(this.document.getCanvas(), this.document.getPoint(Point2D.create(2.5, 1.5)), new CodeCompass.Drawings.RelativeSize(1, 1));
                    ellipse.border.color = Colors.CornflowerBlue;
                    ellipse.fill = Colors.White;
                    ellipse.border.size = 3;
                    var text = new primitives.Text(this.document.getCanvas(), "start", this.document.getPoint(Point2D.create(2, 0.5)), 15, "Verdana", Colors.DarkBlue);
                    text.horizontalAlign = CodeCompass.Drawings.HorizontalAlign.center;
                    this._element = group.add(ellipse, text);
                    this.inputPoints.push(CodeCompass.Drawings.Point2D.create(this.element.size.width / 2, 0));
                    this.outputPoints.push(CodeCompass.Drawings.Point2D.create(this.element.size.width / 2, this.element.size.height));
                };
                return FloydStartElement;
            })(FloydElement);
            Diagram.FloydStartElement = FloydStartElement;            
            var FloydEndElement = (function (_super) {
                __extends(FloydEndElement, _super);
                function FloydEndElement(document) {
                                _super.call(this, document);
                }
                FloydEndElement.prototype.init = function () {
                    _super.prototype.init.call(this);
                    var group = new CodeCompass.Drawings.Raphael2D.Primitives.Group(this.document.getCanvas(), Point2D.Origo, this.document.getSize(this.size));
                    var ellipse = new primitives.Ellipse(this.document.getCanvas(), this.document.getPoint(Point2D.create(2.5, 1.5)), new CodeCompass.Drawings.RelativeSize(0.9, 0.9));
                    ellipse.border.color = Colors.CornflowerBlue;
                    ellipse.fill = Colors.White;
                    ellipse.border.size = 3;
                    var text = new primitives.Text(this.document.getCanvas(), "end", this.document.getPoint(Point2D.create(2, 1)).add(Point2D.create(0, -7.5)), 15, "Verdana", Colors.DarkBlue);
                    text.horizontalAlign = CodeCompass.Drawings.HorizontalAlign.center;
                    this._element = group.add(ellipse, text);
                    this.inputPoints.push(CodeCompass.Drawings.Point2D.create(this.element.size.width / 2, 0));
                    this.outputPoints.push(CodeCompass.Drawings.Point2D.create(this.element.size.width / 2, this.element.size.height));
                };
                return FloydEndElement;
            })(FloydElement);
            Diagram.FloydEndElement = FloydEndElement;            
            var FloydCommand = (function (_super) {
                __extends(FloydCommand, _super);
                function FloydCommand(document, command) {
                                _super.call(this, document);
                    this.command = command;
                }
                FloydCommand.prototype.init = function () {
                    _super.prototype.init.call(this);
                    var group = new CodeCompass.Drawings.Raphael2D.Primitives.Group(this.document.getCanvas(), Point2D.Origo, this.document.getSize(this.size));
                    var rectangle = new primitives.Rectange(this.document.getCanvas(), Point2D.Origo, new CodeCompass.Drawings.RelativeSize(1, 1));
                    rectangle.border.color = Colors.CornflowerBlue;
                    rectangle.fill = Colors.White;
                    rectangle.border.size = 3;
                    var text = new primitives.Text(this.document.getCanvas(), this.command, this.document.getPoint(Point2D.create(2, 0.5)), 15, "Verdana", Colors.DarkBlue);
                    text.horizontalAlign = CodeCompass.Drawings.HorizontalAlign.center;
                    this._element = group.add(rectangle, text);
                    this.inputPoints.push(CodeCompass.Drawings.Point2D.create(this.element.size.width / 2, 0));
                    this.outputPoints.push(CodeCompass.Drawings.Point2D.create(this.element.size.width / 2, this.element.size.height));
                };
                return FloydCommand;
            })(FloydElement);
            Diagram.FloydCommand = FloydCommand;            
            var FloydBlock = (function (_super) {
                __extends(FloydBlock, _super);
                function FloydBlock(document, children) {
                    if (typeof children === "undefined") { children = []; }
                                _super.call(this, document);
                    this._children = children;
                }
                Object.defineProperty(FloydBlock.prototype, "inputPoints", {
                    get: function () {
                        var firstElement = Enumerable.From(this._children).First();
                        if(Ensure.isDefined(firstElement)) {
                            return firstElement.inputPoints;
                        }
                        return [];
                    },
                    enumerable: true,
                    configurable: true
                });
                Object.defineProperty(FloydBlock.prototype, "outputPoints", {
                    get: function () {
                        var lastElement = Enumerable.From(this._children).Last();
                        if(Ensure.isDefined(lastElement)) {
                            return lastElement.outputPoints;
                        }
                        return [];
                    },
                    enumerable: true,
                    configurable: true
                });
                Object.defineProperty(FloydBlock.prototype, "children", {
                    get: function () {
                        return this._children;
                    },
                    enumerable: true,
                    configurable: true
                });
                FloydBlock.prototype.dispose = function () {
                    if(Ensure.isDefined(this._children)) {
                        this._children.forEach(function (e) {
                            return e.dispose();
                        });
                    }
                };
                FloydBlock.prototype.init = function () {
                    _super.prototype.init.call(this);
                    if(Ensure.isDefined(this._children)) {
                        this._children.forEach(function (e) {
                            return e.init();
                        });
                    }
                    this.inputPoints.push(CodeCompass.Drawings.Point2D.create(0, 0));
                    this.outputPoints.push(CodeCompass.Drawings.Point2D.create(0, 0));
                };
                return FloydBlock;
            })(FloydElement);
            Diagram.FloydBlock = FloydBlock;            
            var FloydCondition = (function (_super) {
                __extends(FloydCondition, _super);
                function FloydCondition(document, condition) {
                                _super.call(this, document);
                    this.condition = condition;
                }
                Object.defineProperty(FloydCondition.prototype, "trueCase", {
                    get: function () {
                        return this._trueCase;
                    },
                    set: function (value) {
                        Ensure.defined(value);
                        if(Ensure.hasValue(this._trueCase)) {
                            this._trueCase.dispose();
                        }
                        this._trueCase = value;
                        this._trueCase.init();
                    },
                    enumerable: true,
                    configurable: true
                });
                Object.defineProperty(FloydCondition.prototype, "falseCase", {
                    get: function () {
                        return this._falseCase;
                    },
                    set: function (value) {
                        Ensure.defined(value);
                        if(Ensure.hasValue(this._falseCase)) {
                            this._falseCase.dispose();
                        }
                        this._falseCase = value;
                        this._falseCase.init();
                    },
                    enumerable: true,
                    configurable: true
                });
                FloydCondition.prototype.init = function () {
                    _super.prototype.init.call(this);
                    var group = new CodeCompass.Drawings.Raphael2D.Primitives.Group(this.document.getCanvas(), Point2D.Origo, this.document.getSize(this.size));
                    var rhombus = new primitives.Rhombus(this.document.getCanvas(), Point2D.Origo, new CodeCompass.Drawings.RelativeSize(1, 1));
                    rhombus.border.color = Colors.CornflowerBlue;
                    rhombus.fill = Colors.White;
                    rhombus.border.size = 3;
                    var text = new primitives.Text(this.document.getCanvas(), this.condition, this.document.getPoint(Point2D.create(0, 0)), 15, "Verdana", Colors.DarkBlue);
                    text.horizontalAlign = CodeCompass.Drawings.HorizontalAlign.center;
                    this._element = group.add(rhombus, text);
                    this.inputPoints.push(CodeCompass.Drawings.Point2D.create(this.element.size.width / 2, 0));
                    this.outputPoints.push(CodeCompass.Drawings.Point2D.create(this.element.size.width / 2, this.element.size.height));
                };
                return FloydCondition;
            })(FloydElement);
            Diagram.FloydCondition = FloydCondition;            
            var FloydLoop = (function (_super) {
                __extends(FloydLoop, _super);
                function FloydLoop(document, condition) {
                                _super.call(this, document);
                    this.condition = condition;
                }
                Object.defineProperty(FloydLoop.prototype, "core", {
                    get: function () {
                        return this._core;
                    },
                    set: function (value) {
                        Ensure.defined(value);
                        if(Ensure.hasValue(this._core)) {
                            this._core.dispose();
                        }
                        this._core = value;
                        this._core.init();
                    },
                    enumerable: true,
                    configurable: true
                });
                FloydLoop.prototype.init = function () {
                    _super.prototype.init.call(this);
                    var group = new CodeCompass.Drawings.Raphael2D.Primitives.Group(this.document.getCanvas(), Point2D.Origo, this.document.getSize(this.size));
                    var ellipse = new primitives.Ellipse(this.document.getCanvas(), this.document.getPoint(Point2D.create(2.5, 1.5)), new CodeCompass.Drawings.RelativeSize(1, 1));
                    ellipse.border.color = Colors.CornflowerBlue;
                    ellipse.fill = Colors.White;
                    ellipse.border.size = 3;
                    var text = new primitives.Text(this.document.getCanvas(), this.condition, this.document.getPoint(Point2D.create(2, 0.5)), 15, "Verdana", Colors.DarkBlue);
                    text.horizontalAlign = CodeCompass.Drawings.HorizontalAlign.center;
                    this._element = group.add(ellipse, text);
                    this.inputPoints.push(CodeCompass.Drawings.Point2D.create(this.element.size.width / 2, 0));
                    this.outputPoints.push(CodeCompass.Drawings.Point2D.create(this.element.size.width / 2, this.element.size.height));
                };
                return FloydLoop;
            })(FloydElement);
            Diagram.FloydLoop = FloydLoop;            
            var FloydElementFactory = (function () {
                function FloydElementFactory(document) {
                    this._document = document;
                    this._children = [];
                }
                Object.defineProperty(FloydElementFactory.prototype, "children", {
                    get: function () {
                        return this._children;
                    },
                    enumerable: true,
                    configurable: true
                });
                Object.defineProperty(FloydElementFactory.prototype, "document", {
                    get: function () {
                        return this._document;
                    },
                    enumerable: true,
                    configurable: true
                });
                FloydElementFactory.prototype.updateElement = function (element) {
                    Ensure.defined(element);
                    element.previous = this.lastElement(function (v) {
                        v.next = element;
                    });
                    return element;
                };
                FloydElementFactory.prototype.toBlock = function () {
                    return new FloydBlock(this._document, this._children);
                };
                FloydElementFactory.prototype.lastElement = function (func) {
                    if (typeof func === "undefined") { func = null; }
                    var last = Enumerable.From(this._children).LastOrDefault(undefined);
                    if(Ensure.isDefined(last) && Ensure.isDefined(func)) {
                        func(last);
                    }
                    return last;
                };
                FloydElementFactory.prototype.start = function () {
                    this._children.push(new FloydStartElement(this._document));
                    return this;
                };
                FloydElementFactory.prototype.end = function () {
                    this._children.push(this.updateElement(new FloydEndElement(this._document)));
                    return this;
                };
                FloydElementFactory.prototype.command = function (text) {
                    this._children.push(this.updateElement(new FloydCommand(this._document, text)));
                    return this;
                };
                FloydElementFactory.prototype.loop = function (condition, core) {
                    var factory = new FloydElementFactory(this._document);
                    var loop = new FloydLoop(this._document, condition);
                    loop.core = core(factory).toBlock();
                    loop.core.next = loop;
                    this._children.push(this.updateElement(loop));
                    return this;
                };
                FloydElementFactory.prototype.condition = function (condition, trueCase, falseCase) {
                    var factory = new FloydElementFactory(this._document);
                    var ifThenElse = new FloydCondition(this._document, condition);
                    ifThenElse.trueCase = trueCase(factory).toBlock();
                    ifThenElse.falseCase = falseCase(factory).toBlock();
                    this._children.push(this.updateElement(ifThenElse));
                    return this;
                };
                FloydElementFactory.prototype.block = function (children) {
                    var factory = new FloydElementFactory(this._document);
                    var block = children(factory).toBlock();
                    this._children.push(this.updateElement(block));
                    return this;
                };
                return FloydElementFactory;
            })();
            Diagram.FloydElementFactory = FloydElementFactory;            
        })(Floyd.Diagram || (Floyd.Diagram = {}));
        var Diagram = Floyd.Diagram;
    })(CodeCompass.Floyd || (CodeCompass.Floyd = {}));
    var Floyd = CodeCompass.Floyd;
})(CodeCompass || (CodeCompass = {}));
